#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "selection.h"
#include "iotools.h"



/**
  Constructor initializes data to zero values.
 */
sel::sel()
{
  st = sel_no;
  n = 0;
  ncomp = id1 = NULL;
  rid1 = rid2 = NULL;
  err = 0.0;
  initime = fintime = r = 0.0;
}


/**
  Destructor deallocates used memory.
 */
sel::~sel()
{
  delete [] ncomp;
  delete [] id1;
  delete [] rid1;
  delete [] rid2;
}



/**
  Function reads data of selections from a text file given by parameter in.
  
  @param in - pointer to opened text file

  @retval 0 - on success
  @retval 1 - error reading number of ranges
  @retval 2 - error reading number of ranges
  @retval 3 - error reading range
 */
long sel::read(XFILE *in)
{
  long i;
  char *kwdstr;

  if (xfscanf(in, "%m", &seltype_kwdset, (int *)&st) != 1)
  {
    print_err("cannot read type of selection", __FILE__, __LINE__, __func__);
    return 1;
  }
  switch (st)
  {
    case sel_no:
      n = 0;
      return 0;
    case sel_all:
    case sel_impvalues:
      n = 1;
      id1 = new long [1];
      ncomp = new long [1];
      id1[0] = 0;
      ncomp[0] = 0;
      return 0;
    case sel_period:
      n = 1;
      id1 = new long [1];
      ncomp = new long [1];
      ncomp[0] = 1;
      if (xfscanf(in, "%ld", id1) != 1)
      {
        print_err("cannot read reading period", __FILE__, __LINE__, __func__);
        return 2;
      }
      return 0;
    case sel_realperiod:
      if (xfscanf(in, "%k%le%k%le%k%le%k%le", "ini_time", &initime,
                  "fin_time", &fintime, "period", &r, "err", &err) != 8)
      {
        print_err("Cannot read real period selection", __FILE__, __LINE__, __func__);
        return 2;
      }
      if ((fintime - initime) < 0.0)
      {
        print_err("Final time is less than initial time", __FILE__, __LINE__, __func__);
        return 2;
      }
      n = long((fintime - initime)/r) + 1;
      if (n < 2)
        n = 2;
      rid1 = new double[n];
      memset(rid1, 0, sizeof(*rid1)*n);
      rid1[0] = initime;
      for (i=1; i<n-1; i++)
        rid1[i] += rid1[i-1] + r;
      rid1[n-1] = fintime; 
      return 0;
    case sel_mtx:  
      n = 1;
      id1 = new long [1];
      ncomp = new long [1];
      id1[0] = 0;
      ncomp[0] = 0;
      return 0;
    case sel_range_vec:
    case sel_range_mtx:
      n = 1;
      break;
    case sel_range:
    case sel_realrange:
      kwdstr = "numranges";
      break;
    case sel_list:
    case sel_reallist:
      kwdstr = "numlist_items";
      break;
    default:
      break;
  }
  if ((st != sel_range_vec) && (st != sel_range_mtx))
  {
    if (xfscanf(in, "%k%ld", kwdstr, &n) != 2)
    {
      print_err("cannot read number of ranges or items", __FILE__, __LINE__, __func__);
      return 2;
    }
  }
  if ((st == sel_range) || (st == sel_range_mtx) || (st == sel_range_vec) || (st == sel_list))
  {
    id1 = new long[n];
    memset(id1, 0, sizeof(*id1)*n);
    if ((st ==  sel_range)|| (st == sel_range_mtx) || (st == sel_range_vec))
    {
      ncomp = new long[n];
      memset(ncomp, 0, sizeof(*ncomp)*n);
    }
/*    switch(st)
    {
      case sel_range:
      case sel_range_vec:
      case sel_range_mtx:
        kwdstr = "first_id";
        break;
      case sel_list:
        kwdstr = "item_id";
        break;
      default:
        break;
    }*/
    for (i=0; i<n; i++)
    {
//      if (xfscanf(in, "%k%ld", kwdstr, id1+i) != 2)
      if (xfscanf(in, "%ld", id1+i) != 1)
      {
        print_err("cannot read range or list of selection", __FILE__, __LINE__, __func__);
        return 3;
      }
      id1[i]--;
      if (st == sel_list)
        continue;
//      if (xfscanf(in, "%k%ld", "ncomp", ncomp+i) != 2)
      if (xfscanf(in, "%ld", ncomp+i) != 1)
      {
        print_err("cannot read range of selection", __FILE__, __LINE__, __func__);
        return 3;
      }
    }
  }
  if ((st == sel_realrange) || (st == sel_reallist))
  {
    rid1 = new double[n];
    memset(rid1, 0, sizeof(*rid1)*n);
    if (st ==  sel_realrange)
    {
      rid2 = new double[n];
      memset(rid2, 0, sizeof(*rid2)*n);
    }
/*    switch(st)
    {
      case sel_realrange:
        kwdstr = "first_val";
        break;
      case sel_reallist:
        kwdstr = "item_val";
        break;
      default:
        break;
    }*/
    for (i=0; i<n; i++)
    {
    
//      if (xfscanf(in, "%k%lf", kwdstr, rid1+i) != 2)
      if (xfscanf(in, "%lf", rid1+i) != 1)
      {
        print_err("cannot read real range or list", __FILE__, __LINE__, __func__);
        return 4;
      }
      if (st == sel_reallist)
        continue;
//      if (xfscanf(in, "%k%lf", "sec_val", rid2+i) != 2)
      if (xfscanf(in, "%lf", rid2+i) != 1)
      {
        print_err("cannot read real range in function", __FILE__, __LINE__, __func__);
        return 4;
      }
    }
    if (st == sel_reallist)
    {
//      if (xfscanf(in, "%k%lf", "err", &err) != 2)
      if (xfscanf(in, "%lf", &err) != 1)
      {
        print_err("cannot read real range or list", __FILE__, __LINE__, __func__);
        return 4;
      }
    }
  }
  return 0;
}



/**
  Function prints data of selections to the text file given by the parameter out.
  
  @param in - pointer to opened text file

  @retval 0 - on success
  @retval 1 - error reading number of ranges
  @retval 2 - error reading range
 */
void sel::print(FILE *out)
{
  long i;
  fprintf(out, "%d", (int)st);
  switch (st)
  {
    case sel_no:
    case sel_all:
    case sel_impvalues:
    case sel_mtx:
      fprintf(out, "\n");
      break;
    case sel_range:
      fprintf(out, " %ld\n", n);
      for (i=0; i<n; i++)
        fprintf(out, "%ld %ld\n", id1[i]+1, ncomp[i]);
      break;
    case sel_range_vec:
      for (i=0; i<n; i++)
        fprintf(out, " %ld %ld\n", id1[i]+1, ncomp[i]);
      break;
    case sel_range_mtx:
      for (i=0; i<n; i++)
        fprintf(out, " %ld %ld\n", id1[i]+1, ncomp[i]);
      break;
    case sel_list:
      fprintf(out, " %ld\n", n);
      for (i=0; i<n; i++)
        fprintf(out, "%ld\n", id1[i]+1);
      break;
    case sel_period:
      fprintf(out, " %ld\n", id1[0]);
      break;
    case sel_realperiod:
      fprintf(out, " %le %le %le %le\n", initime, fintime, r, err);
    case sel_realrange:
      fprintf(out, " %ld\n", n);
      for (i=0; i<n; i++)
        fprintf(out, "%e %e\n", rid1[i], rid2[i]);
      break;
    case sel_reallist:
      fprintf(out, " %ld\n", n);
      for (i=0; i<n; i++)
        fprintf(out, "%e\n", rid1[i]);
      fprintf(out, "%e\n", err);
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__); 
  }
}


/**
  Function checks presence of the value id in ranges of the selection.
  
  
  @param id - checked value

  @retval 0 - value was not found
  @retval 1 - value was found
 */
long sel::presence_id(double id)
{
  long i;
  switch (st)
  {
    case sel_realrange:
      for (i=0; i<n; i++)
        if ((id >= rid1[i]) && (id <= rid2[i]))
          return 1;
      break;
    case sel_reallist:
    case sel_realperiod:
      for (i=0; i<n; i++)
        if (fabs(id - rid1[i]) <= err)
          return 1;
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return 0;
}



/**
  Function checks presence of the index id in ranges of the selection.
  
  
  @param id - checked index

  @retval 0 - index was not found
  @retval 1 - index was found
 */
long sel::presence_id(long id)
{
  long i;
  switch (st)
  {
    case sel_no:
      return 0;
    case sel_all:
      return 1;
    case sel_mtx:
      if (id < 6)
        return 1;
      break;
    case sel_range:
    case sel_range_vec:
    case sel_range_mtx:
      for (i=0; i<n; i++)
        if ((id >= id1[i]) && (id < id1[i]+ncomp[i]))
          return 1;
      break;
    case sel_list:
      for (i=0; i<n; i++)
        if (id == id1[i])
          return 1;
      break;
    case sel_period:
      if (id%id1[0] == 0)
        return 1;
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return 0;
}



/**
  Function checks presence of the index id or value rid in ranges of the selection.
  
  
  @param id  - checked index
  @param rid - checked value
  

  @retval 0 - value was not found
  @retval 1 - value was found
 */
long sel::presence_id(long id, double rid)
{
  if ((st == sel_realrange) || (st == sel_reallist) || (st == sel_realperiod))
    return presence_id(rid);
  else
    return presence_id(id);
}



/**
  Function checks presence of the index id or value rid in ranges of the selection
  or checks important times in time controler tc. The function is used
  especially for step selection.
  
  
  @param id  - checked index
  @param rid - checked value
  @param tc  - time controler from the problem description
  

  @retval 0 - value was not found
  @retval 1 - value was found
 */
long sel::presence_id(long id, double rid, timecontr &tc)
{
  if (st == sel_impvalues)
    return tc.iiit;
  if ((st == sel_realrange) || (st == sel_reallist) || (st == sel_realperiod))
    return presence_id(rid);
  else
    return presence_id(id);
}



/**
  Function checks presence of the value id in ranges of the selection.
  
  
  @param id - checked value
  @param ir - output parameter which contains index of range where
              the id was found.

  @retval 0 - index was not found
  @retval 1 - index was found
 */
long sel::presence_id(double id, long &ir)
{
  long i;
  ir = -1;
  switch (st)
  {
    case sel_realrange:
      for (i=0; i<n; i++)
      {
        if ((id >= rid1[i]) && (id <= rid2[i]))
        {
          ir = i;
          return 1;
        }
      }
      break;
    case sel_reallist:
    case sel_realperiod:
      for (i=0; i<n; i++)
      {
        if (fabs(id - rid1[i]) <= err)
        {
          ir = i;
          return 1;
        }
      }
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return 0;
}



/**
  Function checks presence of the index id in ranges of the selection.
  
  
  @param id - checked index
  @param ir - output parameter which contains index of range where
              the id was found.

  @retval 0 - index was not found
  @retval 1 - index was found
 */
long sel::presence_id(long id, long &ir)
{
  long i;
  ir = -1;
  switch (st)
  {
    case sel_no:
      return 0;
    case sel_all:
      ir = 0;
      return 1;
    case sel_mtx:
      if (id < 6)
      {
        ir = 0;
        return 1;
      }
      break;
    case sel_range:
    case sel_range_vec:
    case sel_range_mtx:
      for (i=0; i<n; i++)
      {
        if ((id >= id1[i]) && (id < id1[i]+ncomp[i]))
        {
          ir = i;
          return 1;
        }
      }
      break;
    case sel_list:
      for (i=0; i<n; i++)
      {
        if (id == id1[i])
        {
          ir = i;
          return 1;
        }
      }
      break;
    case sel_period:
      if (id%id1[0] == 0)
      {
        ir = 0;
        return 1;
      }       
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return 0;
}



/**
  Function checks presence of the index id in ranges of the selection.
  Index of ranges have to be greater than ir. It allows for the checking 
  of all overlapping ranges of main selection and corresponding conjugated 
  selections.
  
  
  @param id - checked index
  @param ir - input/output parameter which contains 
              befor the call:
              minimum value of required index of range
              after call :
              index of range where the id was found.

  @retval 0 - index was not found
  @retval 1 - index was found
 */
long sel::presence_idgt(long id, long &ir)
{
  long i;
  switch (st)
  {
    case sel_no:
      return 0;
    case sel_all:
      if (ir < 0)
      {
        ir = 0;
        return 1;
      }
      break;
    case sel_mtx:
      if ((id < 6) && (ir < 0))
      {
        ir = 0;
        return 1;
      }
      break;
    case sel_range:
    case sel_range_vec:
    case sel_range_mtx:
      for (i=0; i<n; i++)
      {
        if ((id >= id1[i]) && (id < id1[i]+ncomp[i]) && (ir < i))
        {
          ir = i;
          return 1;
        }
      }
      break;
    case sel_list:
      for (i=0; i<n; i++)
      {
        if ((id == id1[i]) && (ir < i))
        {
          ir = i;
          return 1;
        }
      }
      break;
    case sel_period:
      if ((id%id1[0] == 0) && (ir < 0))
      {
        ir = 0;
        return 1;
      }       
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return 0;
}



/**
  Function checks presence of the index id in sid-th range of the selection.
  
  
  @param id - checked index
  @param ir - output parameter which contains index of range where
              the id was found.
  @param sid - index of range where the index should be found

  @retval 0 - index was not found
  @retval 1 - index was found
 */
long sel::presence_id(long id, long sid, long &ir)
{
  ir = -1;
  switch (st)
  {
    case sel_no:
      return 0;
    case sel_all:
      if (sid == 0)
      {
        ir = 0;
        return 1;
      }
    case sel_mtx:
      if ((id < 6) && (sid == 0))
      {
        ir = 0;
        return 1;
      }
      break;
    case sel_range:
    case sel_range_vec:
    case sel_range_mtx:
      if ((sid < 0) && (sid >= n))
        break;
      if ((id >= id1[sid]) && (id < id1[sid]+ncomp[sid]))
      {
        ir = sid;
        return 1;
      }
      break;
    case sel_list:
      if ((sid < 0) && (sid >= n))
        break;
      if (id == id1[sid])
      {
        ir = sid;
        return 1;
      }
      break;
    case sel_period:
      if ((id%id1[0] == 0) && (sid == 0))
      {
        ir = 0;
        return 1;
      }       
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return 0;
}



/**
  Function checks presence of the index id or value rid in ranges of the selection.
  
  
  @param id  - checked index
  @param rid - checked value
  @param ir - output parameter which contains index of range where
              the id was found.

  @retval 0 - index was not found
  @retval 1 - index was found
 */
long sel::presence_id(long id, double rid, long &ir)
{
  if ((st == sel_realrange) || (st == sel_reallist) || (st == sel_realperiod))
    return presence_id(rid, ir);
  else
    return presence_id(id, ir);
}



/**
  Function checks presence of the index id in ranges of the selection and
  also checks presence of the index idc in conjugated selections consel.
  Conjugated selections means array of selections whose size is number of 
  ranges of given selection. Each element (i.e. selection) of this array 
  corresponds to one range of given selection.
  
  @param consel - array of conjugated selections
  @param id  - checked index of given selection
  @param idc - checked index of of conjugated selections

  @retval 0 - indices were not found
  @retval 1 - indices were found
 */
long sel::presence_id(sel *consel, long id, long idc)
{
  long ir=-1;
  long i;

  for (i=0; i < n; i++)
  {
    if (presence_idgt(id, ir))
    {
      if (consel[ir].presence_id(idc))
        return 1;
    }
    else
      return 0;
  }
  
  return 0;
}



/** 
  Function computes number of selected items.
  @param tncomp - total number of components which could be selected

  @retval number of selected components

  TKo 9.2008
*/
long sel::give_nselcomp(long tncomp)
{
  long ret=0;
  long i, di;

  switch(st)
  {
    case sel_no:
      ret=0;
      break;
    case sel_all:
      ret=tncomp;
      break;
    case sel_range:
      for(i=0; i<n; i++)
      {
        if (id1[i] >= tncomp) // id is completely out of maximum value
          continue;
        di = id1[i]+ncomp[i]-1; // id of last component in the given range
        if (di < tncomp)
	{
          ret += ncomp[i];  // all components of the given range can be selected
          continue;         
	}
        else
          ret += di+1-tncomp; // only part of range can be selected
      }
      break;
    case sel_list:
      for(i=0; i<n; i++)
      {
        if (id1[i] < tncomp)    ret++;
      }  
      break;
    default:
      print_err("unknown type of selection is required", __FILE__, __LINE__, __func__);
  }
  return ret;
}
