#ifndef INPUT_H
#define INPUT_H

#include <stdio.h>
#include "descrip.h"
#include "xfile.h"
#include "prepalias.h"
#include "alias.h"

/// main function for input from the preprocessor file
long input(XFILE *in, descrip *d);

/// input of description of load cases
long input_lc(XFILE *in);

/// input topolgy using siftop class
long input_siftop(XFILE *in, descrip *d);

/// input material from the material file
long input_materials(char *in);

/// input cross-section from the cross-section file
long input_crs(char *in);

/// input of nodal properties
long input_nodprop(XFILE *in);

/// input of element properties
long input_elemprop(XFILE *in);

/// input of hanging nodes from the separate file
long input_hang_nodes(XFILE *in);

/// assignment of dofs at nodes
long input_nod_ndof(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of boundary conditions at nodes
long input_nod_bocon(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of coupled dofs at nodes
long input_nod_coupl_dofs(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of time functions cotrolling dofs at nodes
long input_nod_dof_tfunc(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of cross-setions at nodes
long input_nod_crsec(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of spring supports at nodes
long input_nod_springs(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of local coordinate systems at nodes
long input_nod_lcs(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of nodal load
long input_nod_load(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of nodal time dependent load
long input_nod_tdload(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of initial conditions at nodes
long input_nod_initcond(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of temparture load at nodes
long input_nod_temper(XFILE *in, const enumstr nodsects[], long nsect);

/// assignment of element types
long input_elem_type(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment of element material types
long input_elem_mat(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment of element cross-section types
long input_elem_crsec(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment of local coordinate systems
long input_elem_lcs(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment of load defined at particular elements
long input_elem_load(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment edge load to elements
long input_elem_loadedge(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment surface load to elements
long input_elem_loadsurf(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment of volume load to elements
long input_elem_loadvol(XFILE *in, const enumstr elemsects[], long nsect);

/// asignment of eigenstrains to elements
long input_elem_eigstr(XFILE *in, const enumstr elemsects[], long nsect);

/// assignment of time functions to elements
long input_elem_eltimefunc(XFILE *in, const enumstr elemsects[], long nsect);

const gentity operator --(gentity &a, int);

long give_lt_id(elloadtype tel);
#endif
