#include "contactmat.h"
#include "matrix.h"
#include "vector.h"
#include "stochdriver.h"
#include "global.h"
#include "intpoints.h"

contactmat::contactmat (void)
{
  slip = 0.0;  normal = 0.0;  limitn=0.0;  limits=0.0;
}
contactmat::~contactmat (void)
{

}

void contactmat::read (XFILE *in)
{
  xfscanf (in,"%lf %lf %lf %lf",&slip,&normal,&limits,&limitn);
}

/**
   function returns stiffness %matrix of material
   
   @param d - stiffness %matrix of the material

   JK, 11.6.2006
*/
void contactmat::matstiff (matrix &d,long ipp)
{
  long n = Mm->ip[ipp].ncompstr;
  vector eps(n),sig(n);

  d[0][0] = slip;
  d[1][1] = normal;
}


/**
   function computes nodal forces in contact problems
   nodal forces are computed from difference of nodal displacements
   at appropriate nodes
   the name nlstresses is used only with respect to other
   "classical" material models which work with stresses
   
   @param ipp - integration point id
   @param im  - index of material type
   @param ido - index in array other
   
   JF, 04/2013
*/
void contactmat::nlstresses (long ipp, long im, long ido)
{
  long i, n = Mm->ip[ipp].ncompstr;
  vector eps(n),sig(n),dpp(n),dpmax(n);
  double dlim;
  
  //  initial values
  for (i=0;i<n;i++)
  {
    eps[i]=Mm->ip[ipp].strain[i];
    dpmax[i]=Mm->ip[ipp].eqother[i];
  }

  // slip
  // maximum elastic deformation
  dlim=limits/slip;

  // division of the deformation
  if (fabs(eps[0]-dpmax[0]) > dlim)  dpp[0]=sgn(eps[0]-dpmax[0])*(fabs(eps[0]-dpmax[0])-dlim);
  if (fabs(eps[0]-dpmax[0]) < dlim)  dpp[0]=0.0;

  // slip stress calculation
  sig[0]=slip*(eps[0]-(dpmax[0]+dpp[0]));

  // normal
  // maximum elastic deformation
  //dlim=limitn/normal;

  sig[1]=eps[1]*normal;
  if (sig[1]>limitn)
    sig[1]=limitn;
  if (sig[1]<-limitn)
    sig[1]=-limitn;
    
/*  // division of the deformation
  if ((dpmax[1] == 0) && (eps[1]<dlim)) dp[1]=0.0;                        // elastic branch
  if                     ((eps[1]-dpmax[1]) > dlim)  dp[1]=eps[1]-dlim;   // plastic branch, adding load
  if ((dpmax[1] > 0)  && ((eps[1]-dpmax[1]) < dlim)) dp[1]=dpmax[1];      // plastic branch, unloading

  // normal stress calculation
  sig[1]=normal*(eps[1]-dp[1]);
*/

  // saving the dp values into "other" array
  for (i=0;i<n;i++) 
  {
    Mm->ip[ipp].other[i] = dpmax[i]+dpp[i];
  }

  

/*  
  sig[1]=eps[1]*normal;
  if (sig[1]>limitn)
    sig[1]=limitn;
  if (sig[1]<-limitn)
    sig[1]=-limitn;
    
  sig[0]=eps[0]*slip;
  if (sig[0]>limits)
    sig[0]=limits;
  if (sig[0]<-limits)
    sig[0]=-limits;
*/  
  for (i=0;i<n;i++){
    Mm->ip[ipp].stress[i]=sig[i];
  }
  
}

void contactmat::changeparam (atsel &atm,vector &val)
{
  long i;
  
  for (i=0;i<atm.num;i++){
    switch (atm.atrib[i]){
    case 0:{
      slip=val[i];
      break;
    }
    case 1:{
      normal=val[i];
      break;
    }
    default:{
      fprintf (stderr,"\n\n wrong number of atribute in function changeparam (%s, line %d).\n",__FILE__,__LINE__);
    }
    }
  }
}

/**
   function copies array other to the array eqother
   
   @param ipp - number of integration point
   @param im  - index of material type
   @param ido - index in array other
   
*/
void contactmat::updateval (long ipp, long im, long ido)
{
  long i,n = Mm->givencompeqother(ipp, im);

  for (i=0;i<n;i++){
    Mm->ip[ipp].eqother[ido+i]=Mm->ip[ipp].other[ido+i];
  }
}

long contactmat::sgn (double a)
{
  long b;
  if (a==0) b=0;
  if (a<0)  b=-1;
  if (a>0)  b=1;
  return (b);
}
