#include "dloadpd.h"
#include "global.h"
#include "intools.h"



/**
  The constructor initializes class attributes to the zero values.

  Created by Tomas Koudelka,
*/
dloadpd::dloadpd()
{
  eq = NULL;
  var = NULL;
  expr[0] = '\0';
  ampl = 0.0;
}



/**
  The destructor deallocates used memory.

  Created by Tomas Koudelka,
*/
dloadpd::~dloadpd()
{
  delete eq;
  delete var;
}



/**
  The function reads data for the dynamical prescribed displacements from the opened text file
  given by the parameter in.

  @param in - pointer to the opened text file

  @retval 0 - on success.
  @retval 1 - error parsing string expression

  Created by Tomas Koudelka,
*/
long dloadpd::read(XFILE *in)
{
  Parser par;
  xfscanf(in, "%le", &ampl);
  xfscanf(in, "%255s", expr);
//  getnexttxt(in->file);
//  fgets(expr, 255, in->file);
  eq = par.TextToTree(expr);
  if (eq == NULL)
  {
    print_err("Error parsing expression", __FILE__, __LINE__, __func__);
    return(1);
  }
  var = eq->Variables.at(0);
  return(0);
}


/**
  The function prints data for the dynamical prescribed displacements into the opened text file
  given by the parameter out.

  @param out - pointer to the opened text file

  @retval 0 - on success.
  @retval 1 - error parsing string expression

  TKr, 08/02/2013 according to read(XFILE *in)
*/
long dloadpd::print(FILE *out)
{
  Parser par;
  fprintf(out, "\n %le",ampl);
  fprintf(out, "\n%255s",expr);
  var = eq->Variables.at(0);//tady promyslet tisk??!!
  return(0);
}



/**
  The function return value of the dynamical prescribed displacement
  for the time given by the parameter t.

  @param t - desired value of time.

  @return The function returns value of the dynamical prescribed displacement for the given time.

  Created by Tomas Koudelka,
*/
double dloadpd::getval(double t)
{
  double ret;
  var->Value = t;
  ret = ampl * eq->Evaluate();
  return(ret);
}
