#ifndef DRPRAG_H
#define DRPRAG_H

#include "iotools.h"
#include "alias.h"
#include "strretalg.h"
struct matrix;
struct vector;
struct atsel;

/**
   This class defines Drucker-Prager plasticity model

   structure of stored data
   plastic strains, consistency parameter, hardening parametr

   28.3.2002
*/

class drprag
{
 public:
  drprag (void);
  ~drprag (void);
  void read (XFILE *in);
  double cohesion(vector &qtr);
  double yieldfunction (matrix &sig, vector &q);
  void deryieldfsigma (matrix &sig, matrix &dfds, vector &q);
  void derpotsigma (matrix &sig, matrix &dgds, vector &q);
  void deryieldfq(vector &qtr, vector &dfq);
  void der_q_gamma(vector &dqdg);
  double plasmodscalar (vector &qtr);
  void updateq(long ipp, vector &epsp, vector &q);
  void matstiff (matrix &d, long ipp, long ido);
  void tangentstiff (matrix &d,matrix &td,long ipp,long ido);
  void nlstresses (long ipp,long im,long ido);
  void nonloc_nlstresses (long ipp, long im, long ido);
  void updateval (long ipp, long im,long ido);
  void giveirrstrains (long ipp, long ido, vector &epsp);
  double give_consparam (long ipp, long ido);
  void changeparam (atsel &atm,vector &val);
  

  ///  friction angle
  double phi;
  ///  cohesion
  double c;
  ///  dilatation
  double psi;
  /// angle of linear hardening/softening
  double theta;
  /// limit cohesion
  double clim;


  double alpha; ///< material constant alpha
  double alpha1; ///< material constant alpha1
  double beta; ///< material constant beta

  ///  stress return algorithm
  strretalg sra;
};

#endif
