#include "elemswitch.h"
#include "genfile.h"
#include "global.h"
#include "elemhead.h"
#include "node.h"
#include "element.h"
#include "intpoints.h"



// *******************
// *******************
// ****  STRAINS  ****
// *******************
// *******************



/**
  The function computes strains at integration points.   
  All components at all integration points are evaluated.
   
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by JK, 28.11.2006
*/
void compute_ipstrains (long lcid)
{
  long i,ne,te;
  
  //  strain array cleaning
  Mm->cleanstrain ();
  
  //  number of elements
  ne=Mt->ne;
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      //  only elements switched on are taken into account
      
      //  type of element
      te = Mt->give_elem_type (i);
      
      switch (te){
	
      case bar2d:{
	Bar2d->res_ip_strains (lcid,i);
	break;
      }
      case bar3d:{
	Bar3d->res_ip_strains (lcid,i);
	break;
      }
      case barq2d:{
	Barq2d->res_mainip_strains (lcid,i);
	break;
      }
      case barq3d:{
	Barq3d->res_mainip_strains (lcid,i);
	break;
      }
	
      case beam2d:{
	Beam2d->nodal_displ (lcid,i);
	break;
      }
      case beam3d:{
	Beam3d->nodal_displ (lcid,i);
	break;
      }
      case beamg3d:{
	Beam3dg->nodal_displ (i,lcid);
	break;
      }
      case subsoilbeam:{
	Sbeam->strains (lcid, i, 0, 0);
	break;
      }
	
      case spring_1:{
	Spring->strains(i, lcid);
	break;
      }
      case spring_2:{
	Spring->strains(i, lcid);
	break;
      }
      case spring_3:{
	Spring->strains(i, lcid);
	break;
      }
      case spring_4:{
	Spring->strains(i, lcid);
	break;
      }
      case spring_5:{
	Spring->strains(i, lcid);
	break;
      }
      case spring_6:{
	Spring->strains(i, lcid);
	break;
      }
	
	
      case planeelementlt:{
	Pelt->res_ip_strains (lcid,i);
	break;
      }
      case planeelementqt:{
	Peqt->res_mainip_strains (lcid,i);
	break;
      }
	
      case planeelementrotlt:{
	Perlt->res_ip_strains (lcid,i);
	break;
      }
	
      case planeelementlq:{
	Pelq->res_ip_strains (lcid,i);
	break;
      }
      case planeelementqq:{
	Peqq->res_ip_strains (lcid,i);
	break;
      }
      case planeelementrotlq:{
	Perlq->res_ip_strains (lcid,i);
	break;
      }
	
      case planeelementsubqt:{
	Pesqt->res_mainip_strains (lcid,i);
	break;
      }
      case planequadcontact:{
	Pqcon->res_mainip_strains (lcid,i);
	break;
      }
	
      case cctel:{
	Cct->res_ip_strains (lcid,i);
	break;
      }
      case dktel:{
	//Dkt->res_mainip_strains (lcid,i);
	Dkt->res_ip_strains (lcid,i);
	break;
      }
      case dstel:{
	Dst->res_ip_strains (lcid,i);
	break;
      }
      case q4plateel:{
	Q4pl->res_ip_strains (lcid,i);
	break;
      }
	
      case subsoilplatetr:{
	break;
      }
	
      case subsoilplateq:{
	Splq->res_mainip_strains (lcid,i);
	break;
      }
	
	
      case shelltrelem:{
	Shtr->res_ip_strains (lcid,i);
	break;
      }
      case shellqelem:{
	Shq->res_ip_strains (lcid,i);
	break;
      }
	
	
      case axisymmlt:{
	Asymlt->res_mainip_strains (lcid,i);
	break;
      }
      case axisymmlq:{
	Asymlq->res_allip_strains (lcid,i);
	break;
      }
      case axisymmqq:{
	Asymqq->res_allip_strains (lcid,i);
	break;
      }
	
	
      case lineartet:{
	Ltet->res_ip_strains (lcid,i);
	break;
      }
      case quadrtet:{
	Qtet->res_ip_strains (lcid,i);
	break;
      }
      case linearhex:{
	Lhex->res_ip_strains (lcid,i);
	break;
      }
      case quadrhex:{
	Qhex->res_ip_strains (lcid,i);
	break;
      }
      case lineartetrot:{
	Ltetrot->res_ip_strains (lcid,i);
	break;
      }
      case linearhexrot:{
	Lhexrot->res_ip_strains (lcid,i);
	break;
      }
	
      case particleelem:{
	break;
      }
      default:{
	print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
      
    }
  }

  //  function modifies strains
  //  eigenstrains are subtracted from the total strains
  //  it works only for the first load case
  Mm->eigstrmod ();
}



/**
  Function computes strains at element nodes.
  Nodal strains are averaged. In case of bar and beam elements 
  be carefull if averaging is reasonable.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  Created by JK, 28.11.2006
*/
void compute_nodestrains (long lcid)
{
  long i;
  elemtype te;
  
  //  for open dx
  Mp->detnodstrain = 1;
  
  for (i=0;i<Mt->nn;i++){
    Mt->nodes[i].nullstrain (lcid);
  }
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      
      switch (te){
	
      case bar2d:{
	Bar2d->nod_strains_ip (lcid, i, 0, 0);
	break;
      }
      case barq2d:{
	Barq2d->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case bar3d:{//zakomentovano pro temelin
	//Bar3d->nod_strains_ip (lcid, i, 0, 0);
	break;
      }
      case barq3d:{//zakomentovano pro temelin
	//Barq3d->nod_strains_ip (lcid,i,0,0);
	break;
      }
	
      case beam2d:{
	Beam2d->nodal_displ (lcid,i);
	break;
      }
      case beam3d:{
	Beam3d->nodal_displ (lcid,i);
	break;
      }

      case spring_1:
      case spring_2:
      case spring_3:
      case spring_4:
      case spring_5:
      case spring_6:
	break;

      case planeelementlt:{
	Pelt->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case planeelementqt:{
	Peqt->nod_strains (lcid,i,0,0);
	break;
      }
      case planeelementrotlt:{
	Perlt->nod_strains_ip (lcid,i,0,0);
	break;
      }
	
      case planeelementlq:{
	Pelq->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case planeelementqq:{
	Peqq->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case planeelementrotlq:{
	//Perlq->nod_strains (lcid,i,0,0);
	break;
      }
	
      case planeelementsubqt:{
	Pesqt->nod_strains (lcid,i,0,0);
	break;
      }
	
      case cctel:{
	Cct->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case dktel:{
	//Dkt->nod_strains (lcid,i,0,0);
	break;
      }
      case dstel:{
	//Dst->nod_strains (lcid,i,0,0);
	break;
      }
      case q4plateel:{
	//Q4pl->nod_strains (lcid,i,0,0);
	break;
      }
	
      case subsoilplatetr:{
	break;
      }
	
      case subsoilplateq:{
	//Splq->nod_strains (lcid,i,0,0);
	break;
      }
	
      case axisymmlt:{
	Asymlt->nod_strains_ip (lcid,i);
	break;
      }
      case axisymmlq:{
	Asymlq->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case axisymmqq:{
	Asymqq->nod_strains_ip (lcid,i);
	break;
      }
	
      case shelltrelem:{
	Shtr->nod_strains_ip (lcid,i);
	break;
      }
      case shellqelem:{
	Shq->nod_strains_ip (lcid,i);
	break;
      }
	
      case lineartet:{
	Ltet->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case quadrtet:{
	Qtet->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case linearhex:{
	Lhex->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case quadrhex:{
	Qhex->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case lineartetrot:{
	Ltetrot->nod_strains_ip (lcid,i,0,0);
	break;
      }
      case linearhexrot:{
	Lhexrot->nod_strains_ip (lcid,i,0,0);
	break;
      }
	
      case particleelem:{
	break;
      }
      default:{
	print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
    }
  }
  
  //  averaging of nodal strains
  for (i=0;i<Mt->nn;i++){
    //if (Gtm->lnso[i]==1){
    Mt->nodes[i].strain_averageval (lcid);
    //}
  }
}



/**
  Function computes strains at element nodes directly and nodal strains are averaged. 
  In case of bar and beam elements be carefull if averaging is reasonable. The function 
  is intended for transfer of mechanical quantities in coupled problems.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  Created by Tomas Koudelka, 14.11.2013
*/
void nodestrains_comp (long lcid)
{
  long i;
  elemtype te;
  
  for (i=0;i<Mt->nn;i++)
    Mt->nodes[i].nullstrain(lcid);
  
  for (i=0;i<Mt->ne;i++)
  {
    if (Gtm->leso[i]==1)
    {
      te = Mt->give_elem_type (i);
      
      switch (te)
      {
        case bar2d:
          Bar2d->nod_strains_comp(lcid, i);
          break;
        case barq2d:
//          Barq2d->nod_strains_comp(lcid,i,0,0);
          break;
        case bar3d:
//          Bar3d->nod_strains_comp(lcid, i, 0, 0);
          break;
        case barq3d:
//          Barq3d->nod_strains_comp(lcid,i,0,0);
          break;
        case spring_1:
        case spring_2:
        case spring_3:
        case spring_4:
        case spring_5:
        case spring_6:
          break;

        case planeelementlt:
//          Pelt->nod_strains_comp(lcid,i,0,0);
          break;
        case planeelementqt:
//          Peqt->nod_strains_comp(lcid,i,0,0);
          break;
	
        case planeelementlq:
//          Pelq->nod_strains_comp(lcid,i,0,0);
          break;
        case planeelementqq:
//          Peqq->nod_strains_comp(lcid,i,0,0);
          break;
	
        case axisymmlt:
//          Asymlt->nod_strains_comp(lcid,i);
          break;
        case axisymmlq:
//          Asymlq->nod_strains_comp(lcid,i,0,0);
          break;
        case axisymmqq:
//          Asymqq->nod_strains_comp(lcid,i);
          break;
	
        case lineartet:
//          Ltet->nod_strains_comp(lcid,i,0,0);
          break;
        case quadrtet:
//          Qtet->nod_strains_comp(lcid,i,0,0);
          break;

        case linearhex:
//          Lhex->nod_strains_comp(lcid,i,0,0);
          break;
     
        case quadrhex:
//          Qhex->nod_strains_comp(lcid,i,0,0);
          break;
        default:
          print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
    }
  }
  
  //  averaging of nodal strains
  for (i=0;i<Mt->nn;i++)
  {
    if (Gtm->lnso[i]==1)
      Mt->nodes[i].strain_averageval (lcid);
  }
}



/**
  The function computes strains at strain points.
   
  @param lcid - load case id
   
  @return The function does not return anything.
 
  Created by JK,
*/
void computestrains (long lcid)
{
  
  //  function computes strains at integration points
  compute_ipstrains (lcid);
  
  /*
  long j,nip,ipp,ncomp,k;
  fprintf (Out,"\n\n\n\n kontrola napeti a sil \n\n");
  for (i=0;i<Mt->ne;i++){
    fprintf (Out,"\n prvek %ld  ",i+1);
    ipp=Mt->elements[i].ipp[0][0];
    fprintf (Out,"ipp  %ld  ",ipp);
    nip=Mt->give_tnip (i);
    fprintf (Out,"nip  %ld  ",nip);
    for (j=0;j<nip;j++){
      fprintf (Out,"\n int. bod %ld  ",j+1);
      ncomp=Mm->ip[ipp].ncompstr;
      for (k=0;k<ncomp;k++){
  fprintf (Out,"  %lf",Mm->ip[ipp].strain[k]);
      }
      ipp++;
    }
  }
  */
  

  /*
  //  number of elements
  ne=Mt->ne;
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      //  type of element
      te = Mt->give_elem_type (i);
      
      switch (te){
      case bar2d:{
  Bar2d->strains (lcid,i,0,0);
  break;
      }
      case bar3d:{
  Bar3d->strains (lcid,i,0,0);
  break;
      }
      case barq2d:{
  Barq2d->strains (lcid,i,0,0);
  break;
      }
      case barq3d:{
  Barq3d->strains (lcid,i,0,0);
  break;
      }
      case beam2d:{
  Beam2d->strains (i,lcid);
  break;
      }
      case beam3d:{
  Beam3d->nodal_displ (i,lcid);
  break;
      }
      case subsoilbeam:{
  Sbeam->strains (lcid, i, 0, 0);
  break;
      }
      case spring_1:
      case spring_2:
      case spring_3:
      case spring_4:
      case spring_5:
      case spring_6:
  {
    Spring->strains (i,lcid);
    break;
  }
  
      case planeelementlt:{
  Pelt->strains (lcid,i,0,0);
  break;
      }
      case planeelementqt:{
  Peqt->strains (lcid,i,0,0);
  break;
      }
      case planeelementrotlt:{
  Perlt->strains (lcid,i,0,0);
  break;
      }
  
      case planeelementlq:{
  Pelq->strains (lcid,i,0,0);
  break;
      }
      case planeelementqq:{
  Peqq->strains (lcid,i,0,0);
  break;
      }
      case planeelementrotlq:{
  Perlq->strains (lcid,i,0,0);
  break;
      }
  
      case planeelementsubqt:{
  Pesqt->strains (lcid,i,0,0);
  break;
      }
  
      case cctel:{
  Cct->strains (lcid,i,0,0);
  break;
      }
      case dktel:{
  Dkt->strains (lcid,i,0,0);
  break;
      }
      case dstel:{
  Dst->strains (lcid,i,0,0);
  break;
      }
      case q4plateel:{
  Q4pl->strains (lcid,i,0,0);
  break;
      }
      case subsoilplateq:{
  Splq->strains (lcid,i,0,0);
  break;
      }
  
  
      case axisymmlt:{
  Asymlt->strains (lcid,i,0,0);
  break;
      }
      case axisymmlq:{
  Asymlq->strains (lcid,i,0,0);
  break;
      }
      case axisymmqq:{
  Asymqq->strains (lcid,i,0,0);
  break;
      }
  
      case shelltrelem:{
  Shtr->strains (lcid,i);
  break;
      }
  
      case lineartet:{
  Ltet->strains (lcid,i,0,0);
  break;
      }
      case quadrtet:{
  Qtet->strains (lcid,i,0,0);
  break;
      }
      case linearhex:{
  Lhex->strains (lcid,i,0,0);
  break;
      }
      case quadrhex:{
  Qhex->strains (lcid,i,0,0);
  break;
      }
  
      case particleelem:{
  break;
      }
  
      default:{
  print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
      
    }
  }
  */
}



// ********************
// ********************
// ****  STRESSES  ****
// ********************
// ********************




/**
  The function computes all stresses at integration points.
   
  @param lcid - load case id
   
  @return The function does not return anything.
 
  Created by JK, 28.11.2006
*/
void compute_ipstresses (long lcid)
{
  long i,ne;
  elemtype te;
  
  //  number of elements
  ne=Mt->ne;
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      //  type of element
      te = Mt->give_elem_type (i);
      
      switch (te){
	
      case bar2d:{
	Bar2d->res_ip_stresses (lcid,i);
	break;
      }
      case bar3d:{
	Bar3d->res_ip_stresses (lcid,i);
	break;
      }
      case barq2d:{
	Barq2d->res_mainip_stresses (lcid,i);
	break;
      }
      case barq3d:{
	Barq3d->res_mainip_stresses (lcid,i);
	break;
      }
	
      case beam2d:{
	Beam2d->nodal_forces (lcid,i);
	break;
      }
	
	
      case beam3d:{
	Beam3d->nodal_forces (lcid,i);
	break;
      }
      case beamg3d:{
	Beam3dg->nodal_forces (lcid,i);
	break;
      }
	
  /*
    case subsoilbeam:{
    Sbeam->strains (lcid, i, 0, 0);
    break;
    }
  */
  
      case spring_1:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_2:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_3:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_4:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_5:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_6:{
	Spring->stresses(i, lcid);
	break;
      }
	
      case planeelementlt:{
	Pelt->res_ip_stresses (lcid,i);
	break;
      }
  /*
    case planeelementqt:{
    Peqt->res_mainip_strains (lcid,i);
    break;
    }
    
  */

      case planeelementrotlt:{
	Perlt->res_ip_stresses (lcid,i);
	break;
      }
	
      case planeelementlq:{
	Pelq->res_ip_stresses (lcid,i);
	break;
      }
	
      case planeelementqq:{
	Peqq->res_ip_stresses (lcid,i);
	break;
      }
	/*
	  case planeelementrotlq:{
	  Perlq->res_mainip_strains (lcid,i);
	  break;
	  }
	  
	  case planeelementsubqt:{
	  Pesqt->res_mainip_strains (lcid,i);
	  break;
	  }
	*/
	
      case planequadcontact:
        Pqcon->res_mainip_stresses(lcid,i);
        break;
      case cctel:{
	Cct->res_ip_stresses (lcid,i);
	break;
      }
	
      case dktel:{
	Dkt->res_ip_stresses (lcid,i);
	break;
      }
      case dstel:{
	Dst->res_ip_stresses (lcid,i);
	break;
      }
      case q4plateel:{
	Q4pl->res_ip_stresses (lcid,i);
	break;
      }
	
      case subsoilplatetr:{
	break;
      }
	
      case subsoilplateq:{
	//Splq->res_mainip_strains (lcid,i);
	break;
      }
	
	
      case shelltrelem:{
	Shtr->res_ip_stresses (lcid,i);
	break;
      }
      case shellqelem:{
	Shq->res_ip_stresses (lcid,i);
	break;
      }
	
      case axisymmlt:{
	Asymlt->res_mainip_stresses (lcid,i);
	break;
      }
      case axisymmlq:{
	Asymlq->res_allip_stresses (lcid,i);
	break;
      }
      case axisymmqq:{
	Asymqq->res_allip_stresses (lcid,i);
	break;
      }
	
      case lineartet:{
	Ltet->res_ip_stresses (lcid,i);
	break;
      }
      case quadrtet:{
	Qtet->res_ip_stresses (lcid,i);
	break;
      }
      case linearhex:{
	Lhex->res_ip_stresses (lcid,i);
	break;
      }
      case quadrhex:{
	Qhex->res_ip_stresses (lcid,i);
	break;
      }
      case lineartetrot:{
	Ltetrot->res_ip_stresses (lcid,i);
	break;
      }
      case linearhexrot:{
	Lhexrot->res_ip_stresses (lcid,i);
	break;
      }
	
      case particleelem:{
	break;
      }
	
      default:{
	print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
    }
  }

  //find_extreme_ipstresses (lcid);
}

/**
   The function searches the minimum and maximum stresses in integration points.
   
   @param lcid - load case id
   
   @return The function does not return anything.
   
   Created by JK, 16. 11. 2012
*/
void find_extreme_ipstresses (long lcid)
{
  long i,ne;
  long nebar2d,nebar3d,nelinhex;
  vector minbar2d,minbar3d,minlinhex;
  vector maxbar2d,maxbar3d,maxlinhex;
  
  if (Bar2d != NULL){
    allocv (Bar2d->tncomp,minbar2d);
    fillv (1.0e80,minbar2d);
    vector maxbar2d(Bar2d->tncomp);
    fillv (-1.0e80,maxbar2d);
  }
  if (Bar3d != NULL){
    allocv  (Bar3d->tncomp,minbar3d);
    fillv (1.0e80,minbar3d);
    allocv (Bar3d->tncomp,maxbar3d);
    fillv (-1.0e80,maxbar3d);
  }
  if (Lhex != NULL){
    allocv (Lhex->tncomp,minlinhex);
    fillv (1.0e80,minlinhex);
    allocv (Lhex->tncomp,maxlinhex);
    fillv (-1.0e80,maxlinhex);
  }
  
  elemtype te;
  
  nebar2d=0;
  nebar3d=0;
  nelinhex=0;
  
  
  //  the number of elements
  ne=Mt->ne;
  
  //  loop over the number of elements
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      //  type of element
      te = Mt->give_elem_type (i);
      
      switch (te){
	
	/*
      case bar2d:{
	Bar2d->res_ip_stresses (lcid,i);
	break;
      }
	*/

      case bar3d:{
	Bar3d->find_extreme_stresses (minbar3d,maxbar3d,lcid,i);
	nebar3d++;
	break;
      }
	/*
      case barq2d:{
	Barq2d->res_mainip_stresses (lcid,i);
	break;
      }
      case barq3d:{
	Barq3d->res_mainip_stresses (lcid,i);
	break;
      }
	
      case beam2d:{
	Beam2d->nodal_forces (lcid,i);
	break;
      }
	
	
      case beam3d:{
	Beam3d->nodal_forces (lcid,i);
	break;
      }
      case beamg3d:{
	Beam3dg->nodal_forces (lcid,i);
	break;
      }
	
      
    case subsoilbeam:{
    Sbeam->strains (lcid, i, 0, 0);
    break;
    }

  
      case spring_1:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_2:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_3:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_4:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_5:{
	Spring->stresses(i, lcid);
	break;
      }
      case spring_6:{
	Spring->stresses(i, lcid);
	break;
      }
	
      case planeelementlt:{
	Pelt->res_ip_stresses (lcid,i);
	break;
      }

    case planeelementqt:{
    Peqt->res_mainip_strains (lcid,i);
    break;
    }
    


      case planeelementrotlt:{
	Perlt->res_ip_stresses (lcid,i);
	break;
      }
	
      case planeelementlq:{
	Pelq->res_ip_stresses (lcid,i);
	break;
      }
	
      case planeelementqq:{
	Peqq->res_ip_stresses (lcid,i);
	break;
      }

	  case planeelementrotlq:{
	  Perlq->res_mainip_strains (lcid,i);
	  break;
	  }
	  
	  case planeelementsubqt:{
	  Pesqt->res_mainip_strains (lcid,i);
	  break;
	  }

	
      case planequadcontact:
        Pqcon->res_mainip_stresses(lcid,i);
        break;
      case cctel:{
	Cct->res_ip_stresses (lcid,i);
	break;
      }
	
      case dktel:{
	Dkt->res_ip_stresses (lcid,i);
	break;
      }
      case dstel:{
	Dst->res_ip_stresses (lcid,i);
	break;
      }
      case q4plateel:{
	Q4pl->res_ip_stresses (lcid,i);
	break;
      }
	
      case subsoilplatetr:{
	break;
      }
	
      case subsoilplateq:{
	//Splq->res_mainip_strains (lcid,i);
	break;
      }
	
	
      case shelltrelem:{
	Shtr->res_ip_stresses (lcid,i);
	break;
      }
      case shellqelem:{
	Shq->res_ip_stresses (lcid,i);
	break;
      }
	
      case axisymmlt:{
	Asymlt->res_mainip_stresses (lcid,i);
	break;
      }
      case axisymmlq:{
	Asymlq->res_allip_stresses (lcid,i);
	break;
      }
      case axisymmqq:{
	Asymqq->res_allip_stresses (lcid,i);
	break;
      }
	
      case lineartet:{
	Ltet->res_ip_stresses (lcid,i);
	break;
      }
      case quadrtet:{
	Qtet->res_ip_stresses (lcid,i);
	break;
      }
	*/
      case linearhex:{
	Lhex->find_extreme_stresses (minlinhex,maxlinhex,lcid,i);
	nelinhex++;
	break;
      }
	/*
      case quadrhex:{
	Qhex->res_ip_stresses (lcid,i);
	break;
      }
      case lineartetrot:{
	Ltetrot->res_ip_stresses (lcid,i);
	break;
      }
      case linearhexrot:{
	Lhexrot->res_ip_stresses (lcid,i);
	break;
      }
	
      case particleelem:{
	break;
      }
	*/
      default:{
	print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
    }
  }
  
  fprintf (stdout,"\n\n\n");
  if (Bar3d != NULL)
    fprintf (stdout,"\n linbar3d  %6ld        % le % le",nebar3d,minbar3d[0],maxbar3d[0]);
  if (Lhex != NULL){
    fprintf (stdout,"\n linhex    %6ld        % le % le % le % le % le % le\n",nelinhex,minlinhex[0],minlinhex[1],minlinhex[2],minlinhex[3],minlinhex[4],minlinhex[5]);
    fprintf (stdout,"                         % le % le % le % le % le % le",maxlinhex[0],maxlinhex[1],maxlinhex[2],maxlinhex[3],maxlinhex[4],maxlinhex[5]);
  }
  
  fprintf (stdout,"\n\n\n");
  
  destrv (minbar2d);
  destrv (minbar3d);
  destrv (minlinhex);

  destrv (maxbar2d);
  destrv (maxbar3d);
  destrv (maxlinhex);
}


/**
  The function computes stresses at element nodes.
  Nodal stresses are averaged. In case of bar and 
  beam elements, be carefull if averaging is reasonable.
  
  @param lcid - load case id
   
  @return The function does not return anything.
 
  Created by JK, 28.11.2006
*/
void compute_nodestresses (long lcid)
{
  long i;
  elemtype te;
  
  for (i=0;i<Mt->nn;i++){
    Mt->nodes[i].nullstress (lcid);
  }
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      
      switch (te){
	
      case bar2d:{
	Bar2d->nod_stresses_ip (lcid, i, 0, 0);
	break;
      }
      case barq2d:{
	Barq2d->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case bar3d:{//zakomentovano pro temelin
	//Bar3d->nod_stresses_ip (lcid, i, 0, 0);
	break;
      }
      case barq3d:{//zakomentovano pro temelin
	//Barq3d->nod_stresses_ip (lcid,i,0,0);
	break;
      }
	
      case beam2d:{
	Beam2d->nodal_forces (lcid,i);
	break;
      }
      case beam3d:{
	Beam3d->nodal_forces (lcid,i);
	break;
      }
      case beamg3d:{
	Beam3dg->nodal_forces (lcid,i);
	break;
      }
	
	
	
      case spring_1:
      case spring_2:
      case spring_3:
      case spring_4:
      case spring_5:
      case spring_6:
	break;
      case planeelementlt:{
	Pelt->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case planeelementqt:{
	Peqt->nod_stresses (lcid,i,0,0);
	break;
      }
      case planeelementrotlt:{
	Perlt->nod_stresses_ip (lcid,i,0,0);
	break;
      }
	
      case planeelementlq:{
	Pelq->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case planeelementqq:{
	Peqq->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case planeelementrotlq:{
	//Perlq->nod_stresses (lcid,i,0,0);
	break;
      }
	
      case planeelementsubqt:{
	Pesqt->nod_stresses (lcid,i,0,0);
	break;
      }
	
      case cctel:{
	Cct->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case dktel:{
	//Dkt->nod_stresses (lcid,i,0,0);
	break;
      }
      case dstel:{
	//Dst->nod_stresses (lcid,i,0,0);
	break;
      }
      case q4plateel:{
	//Q4pl->nod_stresses (lcid,i,0,0);
	break;
      }
	
      case subsoilplatetr:{
	break;
      }
      case subsoilplateq:{
	break;
      }
	
      case axisymmlt:{
	Asymlt->nod_stresses_ip (lcid,i);
	break;
      }
      case axisymmlq:{
	Asymlq->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case axisymmqq:{
	Asymqq->nod_stresses_ip (lcid,i);
	break;
      }
	
      case shelltrelem:{
	Shtr->nod_stresses_ip (lcid,i);
	break;
      }
      case shellqelem:{
	Shq->nod_stresses_ip (lcid,i);
	break;
      }
	
      case lineartet:{
	Ltet->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case quadrtet:{
	Qtet->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case linearhex:{
	Lhex->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case quadrhex:{
	Qhex->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case lineartetrot:{
	Ltetrot->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case linearhexrot:{
	Lhexrot->nod_stresses_ip (lcid,i,0,0);
	break;
      }
      case particleelem:{
	break;
      }
	
      default:{
	print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
      
    }
  }
  
  for (i=0;i<Mt->nn;i++){
    //Mt->nodes[i].strain_averageval (lcid);
    Mt->nodes[i].stress_averageval (lcid);
  }
  
}



/**
  The function computes stresses at stress points.
   
  @param lcid - load case id
   
  @return The function does not return anything.
 
  Created by JK,
*/
void computestresses (long lcid)
{
  long i,ne,te;

  //  all stress components at all integration points
  compute_ipstresses (lcid);

  //  number of elements
  ne=Mt->ne;
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      
      switch (te){
  
      case bar2d:{
  Bar2d->stresses (lcid,i,0,0);
  break;
      }
      case bar3d:{
  Bar3d->stresses (lcid,i,0,0);
  break;
      }
      case barq2d:{
  Barq2d->stresses (lcid,i,0,0);
  break;
      }
      case barq3d:{
  Barq3d->stresses (lcid,i,0,0);
  break;
      }
  
      case beam2d:{
  Beam2d->nodal_forces (lcid,i);
  break;
      }
  
      case beam3d:{
  Beam3d->nodal_forces (lcid,i);
  break;
      }
  
      case spring_1:
      case spring_2:
      case spring_3:
      case spring_4:
      case spring_5:
      case spring_6:
  Spring->stresses(i, lcid);
  break;
      case planeelementlt:{
  Pelt->stresses (lcid,i,0,0);
  break;
      }
      case planeelementqt:{
  Peqt->stresses (lcid,i,0,0);
  break;
      }
      case planeelementrotlt:{
  Perlt->stresses (lcid,i,0,0);
  break;
      }
  
      case planeelementlq:{
  Pelq->stresses (lcid,i,0,0);
  break;
      }
      case planeelementqq:{
  Peqq->stresses (lcid,i,0,0);
  break;
      }
      case planeelementrotlq:{
  Perlq->stresses (lcid,i,0,0);
  break;
      }
  
      case planeelementsubqt:{
  Pesqt->stresses (lcid,i,0,0);
  break;
      }
  
      case cctel:{
  Cct->stresses (lcid,i,0,0);
  break;
      }
      case dktel:{
  Dkt->stresses (lcid,i,0,0);
  break;
      }
  
      case dstel:{
  Dst->stresses (lcid,i,0,0);
  break;
      }
  
      case q4plateel:{
  Q4pl->stresses (lcid,i,0,0);
  break;
      }
  
      case axisymmlt:{
  Asymlt->stresses (lcid,i,0,0);
  break;
      }
      case axisymmlq:{
  Asymlq->stresses (lcid,i,0,0);
  break;
      }
      case axisymmqq:{
  Asymqq->stresses (lcid,i,0,0);
  break;
      }
  
      case shelltrelem:{
  Shtr->stresses (lcid,i);
  break;
      }
      case shellqelem:{
  Shq->stresses (lcid,i);
  break;
      }
  
      case lineartet:{
  Ltet->stresses (lcid,i,0,0);
  break;
      }
      case quadrtet:{
  Qtet->stresses (lcid,i,0,0);
  break;
      }
      case linearhex:{
  Lhex->stresses (lcid,i,0,0);
  break;
      }
      case quadrhex:{
  Qhex->stresses (lcid,i,0,0);
  break;
      }
      case lineartetrot:{
  Ltetrot->stresses (lcid,i,0,0);
  break;
      }
      case linearhexrot:{
  Lhexrot->stresses (lcid,i,0,0);
  break;
      }
      case particleelem:{
  break;
      }
      default:{
  print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
    }
  }
}



/**
  The function computes other values in nodes.
   
  @param lcid - load case id

  @return The function does not return anything.

  Created by JK, 
*/
void compute_nodeothers ()
{
  long i;
  elemtype te;
  
  for (i=0;i<Mt->nn;i++){
    Mt->nodes[i].nullother ();
  }
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      
      switch (te){
	
      case bar2d:{
	Bar2d->nod_eqother_ip (i, 0, 0);
	break;
      }
      case bar3d:{//zakomentovano pro temelin
	//Bar3d->nod_eqother_ip (i, 0, 0);
	break;
      }
      case barq2d:{
	Barq2d->nod_eqother_ip (i, 0, 0);
	break;
      }
      case barq3d:{//zakomentovano pro temelin
	//Barq3d->nod_eqother_ip (i, 0, 0);
	break;
      }
	/*    case beam2d:{
	      Beam2d->stresses (i);
	      break;
	      }
	      case beam3d:{
	      Beam3d->stresses (i);
	      break;
	      }
	      case beam2d:{
	      break;
	      }
	*/
	
      case spring_1:
      case spring_2:
      case spring_3:
      case spring_4:
      case spring_5:
      case spring_6:
	break;
	
      case planeelementlt:{
	Pelt->nod_others (i,0,0);
	break;
      }
	
	/*
	  case planeelementrotlt:{
	  Perlt->nod_others (lcid,i,0,0);
	  break;
	  }
	*/
	
      case planeelementlq:{
	Pelq->nod_eqother_ip (i,0,0);
	break;
      }
	
      case planeelementqq:{
	Peqq->nod_eqother_ip (i,0,0);
	break;
      }
	/*
	  case planeelementrotlq:{
	  Perlq->nod_stresses (lcid,i,0,0);
	  break;
	  }
	  
	  case planeelementsubqt:{
	  Pesqt->nod_stresses (lcid,i,0,0);
	  break;
	  }
	  
	  case cctel:{
	  Cct->nod_stresses (lcid,i,0,0);
	  break;
	  }
	  case dktel:{
	  Dkt->nod_stresses (lcid,i,0,0);
	  break;
	  }
	  case dstel:{
	  Dst->nod_stresses (lcid,i,0,0);
	  break;
	  }
	  case q4plateel:{
	  Q4pl->nod_stresses (lcid,i,0,0);
	  break;
	  }*/
	
      case axisymmlt:{
	Asymlt->nod_eqother_ip (i);
	break;
      }
      case axisymmlq:{
	Asymlq->nod_eqother_ip (i);
	break;
      }
      case axisymmqq:{
	Asymqq->nod_eqother_ip (i);
	break;
      }
	
	/*
	  case shelltrelem:{
	  Shtr->nod_stresses (lcid,i);
	  break;
	  }
	*/
	
      case lineartet:{
	Ltet->nod_eqother_ip (i);
	break;
      }
      case quadrtet:{
	Qtet->nod_eqother_ip (i,0,0);
	break;
      }
      case linearhex:{
	Lhex->nod_eqother_ip (i,0,0);
	break;
      }
      case quadrhex:{
	Qhex->nod_eqother_ip (i,0,0);
	break;
      }
	
      default:{
	print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
      }
    }
  }
  
  for (i=0;i<Mt->nn;i++){
    Mt->nodes[i].other_averageval ();
  }
}



/**
  The function interpolates nodal values to integration points on one element.

  @param eid - element id
  @param nodval - %vector of nodal values
  @param ipval - %vector of values at integration points (output)
   
  @return The function returns interpolated values in the parameter ipval.
 
  Created by JK, 29.11.2006
*/
void elem_intpointval (long eid,vector &nodval,vector &ipval)
{
  elemtype te;

  te = Mt->give_elem_type (eid);

  switch (te){
  case bar2d:{
    Bar2d->intpointval (eid,nodval,ipval);
    break;
  }
  case bar3d:{
    Bar3d->intpointval (eid,nodval,ipval);
    break;
  }
  case barq2d:{
    Barq2d->intpointval (eid,nodval,ipval);
    break;
  }
  case barq3d:{
    Barq3d->intpointval (eid,nodval,ipval);
    break;
  }
    
  case spring_1:{
    Spring->intpointval (eid,nodval,ipval);
    break;
  }
  case spring_2:{
    Spring->intpointval (eid,nodval,ipval);
    break;
  }
  case spring_3:{
    Spring->intpointval (eid,nodval,ipval);
    break;
  }
  case spring_4:{
    Spring->intpointval (eid,nodval,ipval);
    break;
  }
  case spring_5:{
    Spring->intpointval (eid,nodval,ipval);
    break;
  }
  case spring_6:{
    Spring->intpointval (eid,nodval,ipval);
    break;
  }
    
  case planeelementlq:{
    Pelq->intpointval (eid,nodval,ipval);
    break;
  }
  case planeelementqq:{
    Peqq->intpointval (eid,nodval,ipval);
    break;
  }
    
  case axisymmlq:{
    Asymlq->intpointval (eid,nodval,ipval);
    break;
  }
  case axisymmqq:{
    Asymqq->intpointval (eid,nodval,ipval);
    break;
  }
    
  case lineartet:{
    Ltet->intpointval (eid,nodval,ipval);
    break;
  }
  case quadrtet:{
    Qtet->intpointval (eid,nodval,ipval);
    break;
  }
  case linearhex:{
    Lhex->intpointval (eid,nodval,ipval);
    break;
  }
  case quadrhex:{
    Qhex->intpointval (eid,nodval,ipval);
    break;
  }
  case lineartetrot:{
    Ltetrot->intpointval (eid,nodval,ipval);
    break;
  }
  case linearhexrot:{
    Lhexrot->intpointval (eid,nodval,ipval);
    break;
  }

  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}



/**
  The function interpolates nodal values into integration points.

  @param gv - array containing values at all nodes of the mesh
  @param nmq - type of non-mechanical quantity
  @param scale - scale coefficient
   
  @return The function does not return anything.
 
  Created by JK, 21.6.2004, extended by TKr 31/08/2012
*/
void intpointval (double *gv,nonmechquant nmq,double scale)
{
  long i,j,nne,nip,ipid;
  ivector nodes;
  vector nodval,ipval;
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      nne = Mt->give_nne (i);
      nip = Mt->give_tnip (i);
      
      reallocv (nne,nodes);
      reallocv (nne,nodval);
      reallocv (nip,ipval);
      
      //  nodes on element
      Mt->give_elemnodes (i,nodes);
      
      //  nodal values on element
      for (j=0;j<nne;j++){
	nodval[j]=gv[nodes[j]]*scale;
      }
      
      //  interpolation of nodal values to integration points
      elem_intpointval (i,nodval,ipval);
      
      //  number of the first integration point
      ipid=Mt->elements[i].ipp[0][0];
      
      for (j=0;j<nip;j++)
      {
        Mm->storenonmechq(nmq, ipid, ipval[j]);
        ipid++;
      }
    }
  }
}



/**
  Function interpolates nodal values into integration points.
  Order of the used approximation functions is one degree less then on the given element.
   
  @param gv - array containing values at all nodes of the mesh
  @param nmq - type of non-mechanical quantity
   
  @return The function does not return anything.
 
  Created by 21.6.2004, JK, extended by TKr 31/08/2012
*/
void intpointval2 (double *gv,nonmechquant nmq)
{
  long i,j,nne,nip,ipp;
  ivector nodes;
  vector nodval,ipval;
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      nne = Mt->give_nne (i);
      nip = Mt->give_tnip (i);
      
      reallocv (nne,nodes);
      reallocv (nne,nodval);
      reallocv (nip,ipval);
      
      Mt->give_elemnodes (i,nodes);
      
      for (j=0;j<nne;j++){
	nodval[j]=gv[nodes[j]];
      }
      
      elem_intpointval2(i, nodval, ipval);
      
      //  number of the first integration point
      ipp=Mt->elements[i].ipp[0][0];
      
      for (j=0;j<nip;j++)
      {
        Mm->storenonmechq(nmq, ipp, ipval[j]);
        ipp++;
      }
    }
  }
}



/**
  Function interpolates nodal values into integration points.
  Order of the used approximation functions is one degree less then on the given element.
   
  @param eid - element id
  @param nodval - %vector containing values at corner nodes
  @param ipval - %vector of integration point values (output)
   
  @return The function returns resulting integration point values in the %vector ipval.
 
  Created by Tomas Koudelka, 10.12.2013
*/
void elem_intpointval2 (long eid, vector &nodval, vector &ipval)
{
  elemtype te;
  
  te = Mt->give_elem_type (eid);
    
  switch (te)
  {
    case bar2d:
      Bar2d->intpointval (eid,nodval,ipval);
      break;

    case barq2d:
      Barq2d->intpointval2 (eid,nodval,ipval);
      break;

    case barq3d:
      Barq3d->intpointval2 (eid,nodval,ipval);
      break;

    case spring_1:
      Spring->intpointval (eid,nodval,ipval);
      break;

    case spring_2:
      Spring->intpointval (eid,nodval,ipval);
      break;

    case spring_3:
      Spring->intpointval (eid,nodval,ipval);
      break;

    case spring_4:
      Spring->intpointval (eid,nodval,ipval);
      break;

    case spring_5:
      Spring->intpointval (eid,nodval,ipval);
      break;

    case spring_6:
      Spring->intpointval (eid,nodval,ipval);
      break;
	
    case planeelementqq:
      Peqq->intpointval2 (eid,nodval,ipval);
      break;

    case axisymmqq:
      Asymqq->intpointval2 (eid,nodval,ipval);
      break;
	
    case quadrtet:
      Qtet->intpointval2 (eid,nodval,ipval);
      break;
      
    case quadrhex:
      Qhex->intpointval2 (eid,nodval,ipval);
      break;
      
    default:
      print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function interpolates arbitrary quantity defined by its
  nodal values to required inner point.
 
  @param eid - element id
  @param nodval - array containing nodal values
  @param coord - array containing coordinates
   
  @return The function returns required interpolated value.
 
  Created by  JK, 23.2.2002
*/
double interpolate (long eid,double *nodval,double *coord)
{
  long i,nne;
  double val;
  elemtype te;
  vector areacoord(3),volcoord(4),nv;
  
  te = Mt->give_elem_type (eid);
  nne = Mt->give_nne (eid);
  allocv (nne,nv);
  for (i=0;i<nne;i++){
    nv[i]=nodval[i];
  }

  switch (te){
  case planeelementlt:{
    areacoord[0]=coord[0];  areacoord[1]=coord[1];  areacoord[2]=1.0-areacoord[0]-areacoord[1];
    val=Pelt->approx (areacoord,nv);
    break;
  }
  case planeelementqt:{
    areacoord[0]=coord[0];  areacoord[1]=coord[1];  areacoord[2]=1.0-areacoord[0]-areacoord[1];
    val=Peqt->approx (areacoord[0],areacoord[1],nv);
    break;
  }
  case planeelementrotlt:{
    areacoord[0]=coord[0];  areacoord[1]=coord[1];  areacoord[2]=1.0-areacoord[0]-areacoord[1];
    val=Perlt->approx (areacoord,nv);
    break;
  }
  case planeelementlq:{
    val=Pelq->approx (coord[0],coord[1],nv);
    break;
  }
  case planeelementqq:{
    val=Peqq->approx (coord[0],coord[1],nv);
    break;
  }
  case planeelementrotlq:{
    val=Perlq->approx (coord[0],coord[1],nv);
    break;
  }
  case lineartet:{
    volcoord[0]=coord[0];  volcoord[1]=coord[1];  volcoord[2]=coord[2];  volcoord[3]=1.0-volcoord[0]-volcoord[1]-volcoord[2];
    val=Ltet->approx (volcoord,nv);
    break;
  }
  case quadrtet:{
    val=Qtet->approx (coord[0],coord[1],coord[2],nv);
    break;
  }
  case linearhex:{
    val=Lhex->approx (coord[0],coord[1],coord[2],nv);
    break;
  }
  case quadrhex:{
    val=Qhex->approx (coord[0],coord[1],coord[2],nv);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  destrv (nv);
  
  return val;
}



/**
  The function computes volume appropriate to integration points.
   
  @return The function does not return anything.

  Created by JK, 2.3.2004
*/
void ipvolume ()
{
  long i;
  elemtype te;
  
  if (Mm->ipv == NULL)
    delete [] Mm->ipv;
  
  Mm->ipv = new double [Mm->tnip];
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      
      switch (te){
      case bar2d:{
        //Bar2d->ipvolume(i,0,0);
        break;
      }
      case bar3d:{
        //Bar3d->ipvolume(i,0,0);
        break;
      }
      case planeelementlt:{
        Pelt->ipvolume (i,0,0);
        break;
      }
      case planeelementqt:{
        Peqt->ipvolume (i,0,0);
        break;
      }
      case planeelementrotlt:{
        Perlt->ipvolume (i,0,0);
        break;
      }
      case planeelementlq:{
        Pelq->ipvolume (i,0,0);
        break;
      }
      case planeelementqq:{
        Peqq->ipvolume (i,0,0);
        break;
      }
      case planeelementrotlq:{
        Perlq->ipvolume (i,0,0);
        break;
      }

      case lineartet:{
        Ltet->ipvolume (i,0,0);
        break;
      }
      case quadrtet:{
        Qtet->ipvolume (i,0,0);
        break;
      }
      case linearhex:{
        Lhex->ipvolume (i,0,0);
        break;
      }
      case quadrhex:{
        Qhex->ipvolume (i,0,0);
        break;
      }
      case lineartetrot:{
        Ltetrot->ipvolume (i,0,0);
        break;
      }
      case linearhexrot:{
        Lhexrot->ipvolume (i,0,0);
        break;
      }

      default:
        print_err("unknown element type is required", __FILE__, __LINE__, __func__);
      }
    }
  }
}



/**
  The function computes contributions to internal forces from one finite element.
   
  @param i - element id
  @param lcid - load case id
  @param ifor - %vector of internal forces on one element (output)

  @return The function returns contributions in the parameter ifor.

  Created by JK, 3.11.2006
*/
void elem_internal_forces (long i,long lcid,vector &ifor)
{
  elemtype te;
  te = Mt->give_elem_type (i);
  
  switch (te){
    
  case bar2d:{
    Bar2d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case bar3d:{
    Bar3d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case barq2d:{
    Barq2d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case barq3d:{
    Barq3d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case beam2d:{
    Beam2d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case beam3d:{
    Beam3d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case subsoilbeam:{
    Sbeam->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case spring_1:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_2:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_3:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_4:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_5:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_6:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementlt:{
    Pelt->res_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementqt:{
    Peqt->res_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementrotlt:{
    Perlt->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementlq:{
    Pelq->res_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementqq:{
    Peqq->res_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementrotlq:{
    Perlq->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementsubqt:{
    Pesqt->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planequadcontact:{
    Pqcon->res_internal_forces (lcid,i,ifor);
    break;
  }
    
    
  case cctel:{
    Cct->res_internal_forces (lcid,i,ifor);
    break;
  }
  case dktel:{
    Dkt->res_internal_forces (lcid,i,ifor);
    break;
  }
  case dstel:{
    Dst->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case q4plateel:{
    Q4pl->res_internal_forces (lcid,i,ifor);
    break;
  }
  case subsoilplateq:{
    Splq->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case axisymmlt:{
    Asymlt->res_internal_forces (lcid,i,ifor);
    break;
  }
  case axisymmlq:{
    Asymlq->res_internal_forces (lcid,i,ifor);
    break;
  }
  case axisymmqq:{
    Asymqq->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case shelltrelem:{
    Shtr->res_internal_forces (lcid,i,ifor);
    break;
  }
    /*
  case shellqelem:{
    Shq->res_internal_forces (lcid,i,ifor);
    break;
  }
  */
  case lineartet:{
    Ltet->res_internal_forces (lcid,i,ifor);
    break;
  }
  case quadrtet:{
    Qtet->res_internal_forces (lcid,i,ifor);
    break;
  }
  case linearhex:{
    Lhex->res_internal_forces (lcid,i,ifor);
    break;
  }
  case quadrhex:{
    Qhex->res_internal_forces (lcid,i,ifor);
    break;
  }
  case lineartetrot:{
    Ltetrot->res_internal_forces (lcid,i,ifor);
    break;
  }
  case linearhexrot:{
    Lhexrot->res_internal_forces (lcid,i,ifor);
    break;
  }
    
    
  case particleelem:{
    Pelem->res_internal_forces (i,ifor);
    break;
  }
    
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}



/**
  The function computes stresses at one finite element.
  
  @param i - element id
  @param lcid - load case id
   
  @return The function does not return anything but it changes 
          stress array of integration points on the given element.

  Created by Tomas Koudelka, 10.6.2013
*/
void elem_nlstresses(long i,long lcid)
{
  elemtype te;
  te = Mt->give_elem_type (i);
  
  switch (te){
    
  case bar2d:{
    Bar2d->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case bar3d:{
    Bar3d->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case barq2d:{
    Barq2d->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case barq3d:{
    Barq3d->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case beam2d:
  case beam3d:
  case subsoilbeam:
  case spring_1:
  case spring_2:
  case spring_3:
  case spring_4:
  case spring_5:
  case spring_6:
    break;
  case planeelementlt:{
    Pelt->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case planeelementqt:{
    Peqt->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case planeelementrotlt:{
    Perlt->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case planeelementlq:{
    Pelq->compute_nlstress (lcid,i,0,0);
    break;
  }
  case planeelementqq:{
    Peqq->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case planeelementrotlq:{
    Perlq->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case planeelementsubqt:{
    Pesqt->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case planequadcontact:{
    break;
  }
  case cctel:{
    Cct->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case q4plateel:{
    Q4pl->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case subsoilplateq:{
    Splq->compute_nlstress(lcid, i, 0, 0);
    break;
  }
/*
  case subsoilplatetr:{
    Spltr->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case shelltrelem:{
    Shtr->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case shellqelem:{
    Shq->compute_nlstress(lcid, i, 0, 0);
    break;
  }
*/
  case axisymmlt:{
    Asymlt->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case axisymmlq:{
    Asymlq->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case axisymmqq:{
    Asymqq->compute_nlstress(lcid, i, 0, 0);
    break;
  }

  case lineartet:{
    Ltet->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case quadrtet:{
    Qtet->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case linearhex:{
    Lhex->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case quadrhex:{
    Qhex->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case lineartetrot:{
    Ltetrot->compute_nlstress(lcid, i, 0, 0);
    break;
  }
  case linearhexrot:{
    Lhexrot->compute_nlstress(lcid, i, 0, 0);
    break;
  }

  case particleelem:
    break;
  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function computes local values which will be averaged for nonlocal material models
  at one finite element.
  
  @param i - element id
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by Tomas Koudelka, 7.2008
*/
void elem_local_values (long i,long lcid)
{
  elemtype te;
  te = Mt->give_elem_type (i);
  
  switch (te){
    
  case bar2d:{
    Bar2d->local_values(lcid, i, 0, 0);
    break;
  }
  case bar3d:{
    Bar3d->local_values(lcid, i, 0, 0);
    break;
  }
  case barq2d:{
    Barq2d->local_values(lcid, i, 0, 0);
    break;
  }
  case barq3d:{
    Barq3d->local_values(lcid, i, 0, 0);
    break;
  }
  case beam2d:
  case beam3d:
  case subsoilbeam:
  case spring_1:
  case spring_2:
  case spring_3:
  case spring_4:
  case spring_5:
  case spring_6:
    break;
  case planeelementlt:{
    Pelt->local_values(lcid, i, 0, 0);
    break;
  }
  case planeelementqt:{
    Peqt->local_values(lcid, i, 0, 0);
    break;
  }
  case planeelementrotlt:{
    Perlt->local_values(lcid, i, 0, 0);
    break;
  }
  case planeelementlq:{
    Pelq->local_values (lcid,i,0,0);
    break;
  }
  case planeelementqq:{
    Peqq->local_values(lcid, i, 0, 0);
    break;
  }
  case planeelementrotlq:{
    Perlq->local_values(lcid, i, 0, 0);
    break;
  }
  case planeelementsubqt:{
    Pesqt->local_values(lcid, i, 0, 0);
    break;
  }
  case planequadcontact:{
    break;
  }
  case cctel:{
    Cct->local_values(lcid, i, 0, 0);
    break;
  }
  case q4plateel:{
    Q4pl->local_values(lcid, i, 0, 0);
    break;
  }
  case subsoilplateq:{
    Splq->local_values(lcid, i, 0, 0);
    break;
  }
/*
  case subsoilplatetr:{
    Spltr->local_values(lcid, i, 0, 0);
    break;
  }
  case shelltrelem:{
    Shtr->local_values(lcid, i, 0, 0);
    break;
  }
  case shellqelem:{
    Shq->local_values(lcid, i, 0, 0);
    break;
  }
*/
  case axisymmlt:{
    Asymlt->local_values(lcid, i, 0, 0);
    break;
  }
  case axisymmlq:{
    Asymlq->local_values(lcid, i, 0, 0);
    break;
  }
  case axisymmqq:{
    Asymqq->local_values(lcid, i, 0, 0);
    break;
  }

  case lineartet:{
    Ltet->local_values(lcid, i, 0, 0);
    break;
  }
  case quadrtet:{
    Qtet->local_values(lcid, i, 0, 0);
    break;
  }
  case linearhex:{
    Lhex->local_values(lcid, i, 0, 0);
    break;
  }
  case quadrhex:{
    Qhex->local_values(lcid, i, 0, 0);
    break;
  }
  case lineartetrot:{
    Ltetrot->local_values(lcid, i, 0, 0);
    break;
  }
  case linearhexrot:{
    Lhexrot->local_values(lcid, i, 0, 0);
    break;
  }

  case particleelem:
    break;
  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function computes contributions to internal forces from one finite element.
  Contributions are evaluated nonlocally.
   
  @param i - element id
  @param lcid - load case id
  @param ifor - %vector of internal forces on one element (output)

  @return The function returns contributions in the parameter ifor.

  Created by JK, 3.11.2006
*/
void elem_nonloc_internal_forces (long i,long lcid,vector &ifor)
{
  elemtype te;
  te = Mt->give_elem_type (i);
  
  switch (te){
    
  case bar2d:{
    Bar2d->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case bar3d:{
    Bar3d->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case barq2d:{
    Barq2d->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case barq3d:{
    Barq3d->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case beam2d:{
    Beam2d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case beam3d:{
    Beam3d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case subsoilbeam:{
    Sbeam->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case spring_1:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_2:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_3:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_4:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_5:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
  case spring_6:{
    Spring->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementlt:{
    Pelt->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementqt:{
    Peqt->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementrotlt:{
    Perlt->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementlq:{
    Pelq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementqq:{
    Peqq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementrotlq:{
    Perlq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementsubqt:{
    Pesqt->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planequadcontact:{
    Pqcon->res_internal_forces (lcid,i,ifor);
    break;
  }
    
  case cctel:{
    Cct->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
    
  case q4plateel:{
    Q4pl->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case subsoilplateq:{
    Splq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
/*
  case subsoilplatetr:{
    Spltr->lres_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case shelltrelem:{
    Shtr->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case shellqelem:{
    Shq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
*/    

  case axisymmlt:{
    Asymlt->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case axisymmlq:{
    Asymlq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case axisymmqq:{
    Asymqq->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }

  case lineartet:{
    Ltet->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case quadrtet:{
    Qtet->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case linearhex:{
    Lhex->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case quadrhex:{
    Qhex->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case lineartetrot:{
    Ltetrot->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
  case linearhexrot:{
    Lhexrot->res_nonloc_internal_forces (lcid,i,ifor);
    break;
  }
    
  case particleelem:{
    break;
  }
    
  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function computes contributions to increments of internal forces
  from one finite element.
 
  @param i - element id
  @param lcid - load case id
  @param ifor - %vector of increments of internal forces on one element (output)
  
  @return The function returns contributions in the parameter ifor.

  Created by JK, 29.4.2008
*/
void elem_incr_internal_forces (long i,long lcid,vector &ifor)
{
  elemtype te;
  te = Mt->give_elem_type (i);
  
  switch (te){
    
  case bar2d:{
    Bar2d->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case bar3d:{
    Bar3d->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case barq2d:{
    Barq2d->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case barq3d:{
    Barq3d->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  /*
  case beam2d:{
    Beam2d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case beam3d:{
    Beam3d->res_internal_forces (lcid,i,ifor);
    break;
  }
  case subsoilbeam:{
    Sbeam->res_internal_forces (lcid,i,ifor);
    break;
  }

  */  
    
  case spring_1:
  case spring_2:
  case spring_3:
  case spring_4:
  case spring_5:
  case spring_6:
    break;
    
  case planeelementlt:{
    Pelt->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementqt:{
    Peqt->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementrotlt:{
    Perlt->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementlq:{
    Pelq->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementqq:{
    Peqq->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case planeelementrotlq:{
    Perlq->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planeelementsubqt:{
    Pesqt->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
  case planequadcontact:{
    break;
  }
    
  case cctel:{
    Cct->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
  case q4plateel:{
    Q4pl->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case subsoilplateq:{
    Splq->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
  case axisymmlt:{
    Asymlt->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case axisymmlq:{
    Asymlq->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case axisymmqq:{
    Asymqq->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
    /*
  case shelltrelem:{
    Shtr->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  */

  case lineartet:{
    Ltet->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case quadrtet:{
    Qtet->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case linearhex:{
    Lhex->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case quadrhex:{
    Qhex->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case lineartetrot:{
    Ltetrot->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
  case linearhexrot:{
    Lhexrot->res_incr_internal_forces (lcid,i,ifor);
    break;
  }
    
  /*    
  case particleelem:{
    Pelem->res_internal_forces (i,ifor);
    break;
  }
  */
  default:
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  
}



/**
  The function computes nodal forces caused by eigenstrains on one element.
   
  @param lcid - load case id
  @param eid - element id
  @param nfor - %vector of nodal values (output)
  
  @return The function returns contributions in the parameter nfor.
   
  Created by JK,
*/
void elem_eigstrain_forces (long lcid,long eid,vector &nfor)
{
  switch (Mt->give_elem_type(eid)){
    
  case bar2d:{
    Bar2d->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case bar3d:{
    Bar3d->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case barq2d:{
    Barq2d->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case barq3d:{
    Barq3d->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }

  case planeelementlt:{
    Pelt->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case planeelementqt:{
    Peqt->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case planeelementrotlt:{
    Perlt->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }

  case planeelementlq:{
    Pelq->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case planeelementqq:{
    Peqq->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case planeelementrotlq:{
    Perlq->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case planeelementsubqt:{
    Pesqt->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case planequadcontact:{
    break;
  }

  case cctel:{
    Cct->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case q4plateel:{
    Q4pl->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case subsoilplateq:{
    Splq->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }


  case axisymmlt:{
    Asymlt->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case axisymmlq:{
    Asymlq->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case axisymmqq:{
    Asymqq->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
    
  case lineartet:{
    Ltet->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case quadrtet:{
    Qtet->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case linearhex:{
    Lhex->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case quadrhex:{
    Qhex->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case lineartetrot:{
    Ltetrot->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }
  case linearhexrot:{
    Lhexrot->res_eigstrain_forces (lcid,eid,nfor);
    break;
  }

  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function computes stiffness %matrix of required element.

  @param lcid - load case id
  @param eid - element id
  @param sm - stiffness %matrix of the element (output)

  @return The function returns assembled %matrix in the parameter sm.

  Created by JK,
*/
void stiffmat (long lcid,long eid,matrix &sm)
{
  elemtype te;
  
  te = Mt->give_elem_type (eid);

  switch (te){

  case bar2d:{
    Bar2d->res_stiffness_matrix (eid,sm);
    break;
  }
  case bar3d:{
    Bar3d->res_stiffness_matrix (eid,sm);
    break;
  }
  case barq2d:{
    Barq2d->res_stiffness_matrix (eid,sm);
    break;
  }
  case barq3d:{
    Barq3d->res_stiffness_matrix (eid,sm);
    break;
  }
  case beam2d:{
    Beam2d->res_stiffness_matrix (eid,sm);
    break;
  }
  case beam3d:{
    Beam3d->res_stiffness_matrix (eid,sm);
    break;
  }
  case beamg3d:{
    Beam3dg->res_stiffness_matrix (eid,sm);
    break;
  }
  case subsoilbeam:{
    Sbeam->res_stiffness_matrix (eid,sm);
    break;
  }

  case spring_1:{
    Spring->res_stiffness_matrix (eid,sm);
    break;
  }
  case spring_2:{
    Spring->res_stiffness_matrix (eid,sm);
    break;
  }
  case spring_3:{
    Spring->res_stiffness_matrix (eid,sm);
    break;
  }
  case spring_4:{
    Spring->res_stiffness_matrix (eid,sm);
    break;
  }
  case spring_5:{
    Spring->res_stiffness_matrix (eid,sm);
    break;
  }
  case spring_6:{
    Spring->res_stiffness_matrix (eid,sm);
    break;
  }

  case planeelementlt:{
    Pelt->res_stiffness_matrix (eid,sm);
    break;
  }
  case planeelementqt:{
    Peqt->res_stiffness_matrix (eid,sm);
    break;
  }
  case planeelementrotlt:{
    Perlt->res_stiffness_matrix (eid,sm);
    break;
  }

  case planeelementlq:{
    Pelq->res_stiffness_matrix (lcid,eid,sm);
    break;
  }
  case planeelementqq:{
    Peqq->res_stiffness_matrix (eid,sm);
    break;
  }
  case planeelementrotlq:{
    Perlq->res_stiffness_matrix (eid,sm);
    break;
  }
    
  case planeelementsubqt:{
    Pesqt->res_stiffness_matrix (eid,sm);
    break;
  }

  case planequadcontact:{
    Pqcon->res_stiffness_matrix (eid,sm);
    break;
  }
    
  case cctel:{
    Cct->res_stiffness_matrix (eid,sm);
    break;
  }
  case dktel:{
    Dkt->res_stiffness_matrix (eid,sm);
    break;
  }
  case dstel:{
    Dst->res_stiffness_matrix (eid,sm);
    break;
  }
  case q4plateel:{
    Q4pl->res_stiffness_matrix (eid,sm);
    break;
  }
  case argyristr:{
    //Argtr->res_stiffness_matrix (eid,sm);
    Argtrpl->res_stiffness_matrix (eid,sm);
    break;
  }
    
  case subsoilplatetr:{
    Spltr->res_stiffness_matrix (eid,sm);
    break;
  }
  case subsoilplateq:{
    Splq->res_stiffness_matrix (eid,sm);
    break;
  }
    
  case axisymmlt:{
    Asymlt->res_stiffness_matrix (eid,sm);
    break;
  }
  case axisymmlq:{
    Asymlq->res_stiffness_matrix (eid,sm);
    break;
  }
  case axisymmqq:{
    Asymqq->res_stiffness_matrix (eid,sm);
    break;
  }

  case shelltrelem:{
    Shtr->res_stiffness_matrix (eid,sm);
    break;
  }
  case shellqelem:{
    Shq->res_stiffness_matrix (eid,sm);
    break;
  }

  case lineartet:{
    Ltet->res_stiffness_matrix (eid,sm);
    break;
  }
  case quadrtet:{
    Qtet->res_stiffness_matrix (eid,sm);
    break;
  }
  case linearhex:{
    Lhex->res_stiffness_matrix (lcid,eid,sm);
    break;
  }
  case quadrhex:{
    Qhex->res_stiffness_matrix (eid,sm);
    break;
  }
  case lineartetrot:{
    Ltetrot->res_stiffness_matrix (eid,sm);
    break;
  }
  case linearhexrot:{
    Lhexrot->res_stiffness_matrix (lcid,eid,sm);
    break;
  }
  case linearwed:{
    Lwed->res_stiffness_matrix (eid,sm);
    break;
  }
  case quadrwed:{
    Qwed->res_stiffness_matrix (eid,sm);
    break;
  }
    
  case particleelem:{
    Pelem->res_stiffness_matrix (eid,sm);
    break;
  }
  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function computes mass %matrix of required element.

  @param lcid - load case id
  @param eid - element id
  @param mm - mass %matrix of the element (output)

  @return The function returns assembled %matrix in the parameter mm.

  Created by JK,
*/
void massmat (long lcid,long eid,matrix &mm)
{
  elemtype te;
  
  te = Mt->give_elem_type (eid);

  switch (te){

  case bar2d:{
    Bar2d->mass_matrix (eid,mm);
    break;
  }
    /*
  case barq2d:{
    Barq2d->res_mass_matrix (eid,mm);
    break;
  }
    */
  case beam2d:{
    Beam2d->res_mass_matrix (eid,mm);
    break;
  }
  case bar3d:{
    Bar3d->res_mass_matrix (eid,mm);
    break;
  }
  case beam3d:{
    Beam3d->res_mass_matrix (eid,mm);
    break;
  }

  case spring_1:
  case spring_2:
  case spring_3:
  case spring_4:
  case spring_5:
  case spring_6:
    break;
  
  case planeelementlt:{
    Pelt->res_mass_matrix (eid,mm);
    break;
  }
  case planeelementqt:{
    Peqt->mass_matrix (eid,mm);
    break;
  }
  case planeelementrotlt:{
    Perlt->res_mass_matrix (eid,mm);
    break;
  }
  case planeelementlq:{
    Pelq->res_mass_matrix (eid,mm);
    break;
  }
  case planeelementqq:{
    Peqq->res_mass_matrix (eid,mm);
    break;
  }
  case planeelementrotlq:{
    Perlq->res_mass_matrix (eid,mm);
    break;
  }

  case planeelementsubqt:{
    Pesqt->res_mass_matrix (eid,mm);
    break;
  }

  case cctel:{
    Cct->res_mass_matrix (eid,mm);
    break;
  }

  case axisymmlt:{
    Asymlt->mass_matrix (eid,mm);
    break;
  }
  case axisymmlq:{
    Asymlq->mass_matrix (eid,mm);
    break;
  }
  case axisymmqq:{
    Asymqq->mass_matrix (eid,mm);
    break;
  }

  case lineartet:{
    Ltet->mass_matrix (eid,mm);
    break;
  }
  case quadrtet:{
    Qtet->mass_matrix (eid,mm);
    break;
  }
  case linearhex:{
    Lhex->mass_matrix (eid,mm);
    break;
  }
  case quadrhex:{
    Qhex->mass_matrix (eid,mm);
    break;
  }
  case lineartetrot:{
    Ltetrot->mass_matrix (eid,mm);
    break;
  }
  case linearhexrot:{
    Lhexrot->mass_matrix (eid,mm);
    break;
  }

  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function computes initial stiffness %matrix of required element.
   
  @param lcid - load case id
  @param eid - element id
  @param sm - initial stiffness %matrix of the element (output)

  @return The function returns assembled %matrix in the parameter sm.

  Created by JK,
*/
void initstiffmat (long lcid,long eid,matrix &sm)
{
  elemtype te;
  
  te = Mt->give_elem_type (eid);

  switch (te){

  case beam2d:{
    Beam2d->initstr_matrix_expl (eid,0,0,sm);
    break;
  }

  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }

}



/**
  The function computes load %matrix of required element.
   
  @param eid - element id
  @param sm - load %matrix of the element
   
  @return The function returns assembled %matrix in the parameter lm.

  Created by JK, 12.10.2008
*/
void loadmat (long eid,matrix &lm)
{
  switch (Mt->give_elem_type (eid)){
  case bar2d:{
    Bar2d->res_load_matrix (eid,lm);
    break;
  }
  case beam3d:{
    Beam3d->load_matrix (eid,lm);
    break;
  }
    
  case planeelementlt:{
    Pelt->res_load_matrix (eid,lm);
    break;
  }
  case planeelementqt:{
    Peqt->load_matrix (eid,lm);
    break;
  }
  case planeelementlq:{
    Pelq->res_load_matrix (eid,lm);
    break;
  }
  case planeelementqq:{
    Peqq->res_load_matrix (eid,lm);
    break;
  }
  case planeelementrotlt:{
    Perlt->load_matrix (eid,lm);
    break;
  }
    
  case cctel:{
    Cct->load_matrix (eid,lm);
    break;
  }
  case q4plateel:{
    Q4pl->load_matrix (eid,lm);
    break;
  }
    
  case axisymmlt:{
    Asymlt->load_matrix (eid,lm);
    break;
  }
  case axisymmlq:{
    Asymlq->load_matrix (eid,lm);
    break;
  }
  case axisymmqq:{
    Asymqq->load_matrix (eid,lm);
    break;
  }
    
  case lineartet:{
    Ltet->load_matrix (eid,lm);
    break;
  }
  case quadrtet:{
    Qtet->load_matrix (eid,lm);
    break;
  }
  case linearhex:{
    Lhex->load_matrix (eid,lm);
    break;
  }
  case quadrhex:{
    Qhex->load_matrix (eid,lm);
    break;
  }
  default:
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function returns non-transport=mechanical quantities at element nodes. Nodal
  values are copied from the closest integration point on element.
  It is used for passing required mechanical quantities by TRFEL in coupled problems.

  @param eid - element id
  @param nodval - %vector of nodal values
  @param qt - type of mechanical quantity
   
  @return The function does not return anything but it stores nodal values in parameter nodval.
 
  Created 12/06/2012 TKr
  Modified by Tomas Koudelka 9.10.2013
*/
void elem_mechq_nodval(long eid, vector &nodval, nontransquant qt)
{
  elemtype te;

  te = Mt->give_elem_type (eid);

  switch (te){
  case bar2d:{
    Bar2d->mechq_nodval (eid,nodval,qt);
    break;
  }
  case planeelementlq:{
    Pelq->mechq_nodval (eid,nodval,qt);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}



/**
  The function returns non-transport=mechanical quantities from element nodes.
  Nodal values are computed directly at nodes according to strains.
  It is used for passing required mechanical quantities by TRFEL in coupled problems.

  @param eid - element id
  @param nodval - %vector of nodal values of all required quantities, i.e., 
                  nodal value of i-th quantity in j-th node is given by nodval[i*nne+j] where nne 
                  is the number of nodes on eid-th element.
  @param ncne - number of computed nodes on element (only first ncne of nodes is calculated)
  @param nq - number of required mechanical quantities
  @param qt - array of types of required mechanical quantities
   
  @return The function does not return anything but it stores nodal values in parameter nodval.
 
  Created 12/06/2012 TKr
  Modified by Tomas Koudelka 9.10.2013
*/
void elem_mechq_nodval_comp(long eid, vector &nodval, long ncne, long nq, nontransquant *qt)
{
  elemtype te;

  te = Mt->give_elem_type (eid);

  switch (te){
  case bar2d:{
    Bar2d->mechq_nodval_comp (eid,nodval,ncne,nq,qt);
    break;
  }
  case planeelementlq:{
    Pelq->mechq_nodval_comp (eid,nodval,ncne,nq,qt);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}

