#ifndef MECHMAT_H
#define MECHMAT_H


#include <stdio.h>
#include "alias.h"
#include "aepoints.h"
#include "iotools.h"
#include "selection.h"

class intpoints;
class elastisomat;
class elastgmat3d;
class splas1d;
class j2flow;
class j2flow2;
class microM4;
class microSIM;
class microfiber;
class mohrcoulomb;
class mohrcoulombpar;
class boermat;
class drprag;
class camclay;
class camclaycoup;
class shefplast;
class hissplas;
class glasgmech;
class glasgowdam;
class creepdam;
class timeswmat;
class effstress;
class svisplas;
class simviscous;
class lemaitre;
class scaldam;
class scaldamcc;
class ortodam;
class ortodamrot;
class anisodam;
class anisodamrot;
class aci78;
class cebfip78;
class graphmat;
class varelastisomat;
class b3mat;
class rspecmat;
class dplmat;
class creepb;
class creep_effym;
class consol;
class winpast;
class nonlocmicroM4;
struct matrix;
struct vector;
class therisomat;
class nonlocplast;
class nonlocdamg;
class damplast;
class geoelastmat;
class visplast;
class chen;
class lenjonesmat;
class contactmat;
class cebfipcontactmat;
class relaxeuroc;
class shrinkmat;
class hypoplast;


/**
  Class mechmat:
   
  It is one of the 5 most important classes of the program.
  (probdesc, mechtop, mechmat, mechbclc, mechcrsec)
   
  Class mechmat contains data about materials in problem.
   
  basic data:
   - nip (long) - stands for number of all integration points for stiffness matrix computation in problem
   - nmt (long) - stands for number of material types
   
  Class mechmat creates nip objects of class intpoints.
   
  Created by JK, TKo
*/
class mechmat
{
 public:
  mechmat ();
  ~mechmat ();

  /// determines number of integration points and sets integration point pointers on elements
  long intpnum (void);

  /// allocation of array of integration points
  void intpointalloc ();

  /// allocation of array of intial conditions
  void alloc_ic ();

  /// allocation of array of non-mechanical quantities
  void alloc_nonmechq(long n);

  /// obsolate function for reading material properties on individual integration points
  void readip (XFILE *in);

  /// zero stage of int. point intialization - copying material ids from elements to int. points
  void intpointinit ();

  /// the first stage of int. point intialization - setting of ncompstr, ssst according to used elements
  void init_ip_1 (void);

  /// the second stage of int. point intialization - allocation of strain/stress/eqother/other on int. points
  void init_ip_2 (void);

  /// reading of materials and their parameters
  void read (XFILE *in);

  /// reading of materials and their parameters
  void readmatchar (XFILE *in);

  /// printing of materials
  void print (FILE *out);

  /// printing of individual materials and their parameters
  void printmatchar (FILE *out);

  /// allocates array for eigenstrain storage
  void alloceigstrains ();

  /// stes eigenstrains to zero
  void nulleigstrains ();

  /// allocates array for thermal strain storage
  void alloctempstrains ();

  /// sets thermal strains to zero
  void nulltempstrains ();

  /// allocates array for eigenstress storage set it to zero
  void alloceigstresses ();

  /// initailizes material models for all int. points before begining of main computation procedure
  void initmaterialmodels (long lcid);

  /// initailizes material models for the given int. point
  void initvalues (long lcid, long ipp,long im,long ido);



  // *****************************************************************************
  // Function used for access to stress/strain/eqother/other arrays on int. points
  // *****************************************************************************

  /// storage of strains on given int. point for given load case
  void storestrain (long lcid,long ipp,vector &eps);

  /// storage of strain components from given index on int. point for given load case
  void storestrain (long lcid,long ipp,long fi,vector &eps);

  /// storage of given number of strain components from given index on int. point for given load case
  void storestrain (long lcid,long ipp,long fi,long ncomp,vector &eps);

  /// storage of given number of strain components from given index on int. point for given load case
  void storestrain (long lcid, long ipp, long fi, long ncomp, double *eps);

  /// returns strains on given int. point for given load case
  void givestrain (long lcid,long ipp,vector &eps);

  /// returns strain components from given index on int. point for given load case
  void givestrain (long lcid,long ipp,long fi,vector &eps);

  /// retruns given number of strain components from given index on int. point for given load case
  void givestrain (long lcid,long ipp,long fi,long ncomp,vector &eps);

  /// sets strains to zero
  void cleanstrain ();

  /// storage of stresses on given int. point for given load case
  void storestress (long lcid,long ipp,vector &sig);

  /// storage of stress components from given index on int. point for given load case
  void storestress (long lcid,long ipp,long fi,vector &sig);

  /// storage of stress components from given index on int. point for given load case
  void storestress (long lcid,long ipp,long fi,long ncomp,vector &sig);

  /// storage of stress components from given index on int. point for given load case
  void storestress (long lcid,long ipp,long fi,long ncomp,double *sig);

  /// returns stresses on given int. point for given load case
  void givestress (long lcid,long ipp,vector &sig);

  /// returns stress components from given index on int. point for given load case
  void givestress (long lcid,long ipp,long fi,vector &sig);

  /// retruns given number of stress components from given index on int. point for given load case
  void givestress (long lcid,long ipp,long fi,long ncomp,vector &sig);

  /// storage of eigenstrains for given int. point
  void storeeigstrain (long ipp,vector &eps);

  /// store given eigenstrain components for given int. point
  void storeeigstrain (long ipp,long fi,long ncomp,vector &eps);

  /// retruns eigenstrains for given int. point
  void giveeigstrain (long ipp,vector &eps);

  /// retruns given eigenstrain components for given int. point
  void giveeigstrain (long ipp,long fi,long ncomp,vector &eps);

  /// storage of eigenstresses for given int. point
  void storeeigstress (long ipp,vector &sig);

  /// storage of given eigenstress components on given int. point
  void storeeigstress (long ipp,long fi,long ncomp,vector &sig);

  /// returns eigenstresses for given int. point
  void giveeigstress (long ipp,vector &sig);

  /// returns eigenstress components from given id on given int. point
  void giveeigstress (long ipp,long fi,vector &sig);

  /// returns given eigenstress components on given int. point
  void giveeigstress (long ipp,long fi,long ncomp,vector &sig);
  
  /// storage of given internal variable components on given int. point
  void storeeqother (long ipp,long fi,long ncomp,double *comp);

  /// returns given internal variable components on given int. point
  void giveeqother (long ipp,long fi,long ncomp,double *comp);

  /// storage of given internal variable components on given int. point
  void storeother (long ipp,long fi,long ncomp,double *comp);

  /// returns the number of other array components on given int. point (num. of internal variables used by material models)
  long givencompother (long ipp,long im);

  /// returns the number of eqother array components on given int. point (num. of internal variables used by material models)
  long givencompeqother (long ipp,long im);

  /// storage of given nonlocal values on given integration point
  void storenonloc (long ipp,long fi,long ncomp,double *comp);

  /// returns given nonlocal values from integration point
  void givenonloc (long ipp,long fi,long ncomp,double *comp);



  // ******************************************************************
  // Functions for retrieving of mechanical quantities from int. points
  // ******************************************************************

  /// returns tensor quantity for integral \int B^T \mbf{q} dV on element (nodal force computation)
  void givequantity (integratedquant iq,long lcid,long ipp,long fi,vector &ipv);

  /// returns actual Young modulus for given integration point
  double give_actual_ym(long ipp, long im=0, long ido=0);

  /// returns initial Young modulus for given integration point  
  double give_initial_ym(long ipp, long im=0, long ido=0);

  /// returns actual Poissons ratio for given integration point  
  double give_actual_nu(long ipp, long im=0, long ido=0);

  /// returns initial Poissons ratio for given integration point  
  double give_initial_nu(long ipp, long im=0, long ido=0);

  /// returns actual tensile strength (for damage models)
  double give_actual_ft (long ipp, long im=0, long ido=0);

  /// returns actual compression strength (for damage models)
  double give_actual_fc (long ipp, long im=0, long ido=0);

  /// returns required mechanical quantity for passed to TRFEL
  double givemechq(nontransquant qt, long ipp);

  /// returns preconsolidation pressure for Cam-Clay type models
  double give_preconspress(long ipp, long im=0, long ido=0);

  /// returns virgin porostity for Cam-Clay type models
  double give_virgporosity(long ipp, long im=0, long ido=0);

  /// returns initial porosity for Cam-Clay type models
  double give_iniporosity(long ipp, long im=0, long ido=0);

  /// returns consistency parameter for plasticity models
  double give_consparam (long ipp, long im=0, long ido=0);

  /// returns damage parameter omega
  double give_dampar (long ipp,long im=0,long ido=0);

  /// returns length of process zone
  double give_proczonelength (long ipp,long im=0,long ido=0);

  /// returns crack width
  double give_crackwidth (long ipp,long im=0,long ido=0);

  /// returns equivalent strain for damage models
  void give_kappa(long ipp,long im,long ido, vector &kappa);

  /// return irreversible strains for nonlinear material models
  void giveirrstrains(long ipp, long im, long ido, vector &epsirr);




  // **********************************************************************
  // Functions for retrieving of non-mechanical quantities from int. points
  // **********************************************************************
  
  /// searches for the number and types of required non-mechanical quantities at all material models
  long search_reqnmq(nonmechquant* &rnmq);

  /// marks required types of non-mechanical quantities by the given material model in the given int. point
  void give_reqnmq(long ipp, long im, long *anmq);
  
  /// returns given non-mechanical quantity from nonmechq array on given integration point
  double givenonmechq (nonmechquant qt, long ipp);

  /// stores given non-mechanical quantity from nonmechq array on given integration point
  void   storenonmechq (nonmechquant qt, long ipp, double val);

  /// returns status of non-mechanical quantity (stored in nonmechq 1, otherwise 0)
  long   givestatusnmq (nonmechquant qt);

  /// returns status of non-mechanical quantity (stored in nonmechq 1, otherwise 0)
  long   givenonmechqid (nonmechquant qt);




  // *******************************************
  // Functions connected with stress calculation
  // *******************************************
  
  /// computes the stiffness matrix for the given integration point and material
  void matstiff (matrix &d,long ipp,long im=0,long ido=0);

  /// computes elastic stiffness matrix
  void elmatstiff (matrix &d,long ipp);

  /// computes elastic complience matrix
  void elmatcompl (matrix &c,long ipp);

  /// computes eigenstress and corresponding eigenstrains defined by relaxation material models
  void eigenstresses (vector &sig,long ipp,long id);

  /// computes stresses with respect to material model and store them in the given int. point

  void computenlstresses (long ipp, long im=0, long ido=0);

  /// computes stress increments with respect to material model and store them in the given int. point
  void computenlstressesincr (long ipp, long im=0, long ido=0);

  /// computes stress with respect to non-local material model and store them in the given int. point
  void compnonloc_nlstresses (long ipp, long im=0, long ido=0);

  /// compute averaged values of strains //???!!! candidate for removal
  void compute_averstrains ();

  /// compute strains due to temperature changes (thermal strains)
  void temprstrains (long lcid);

  /// subtracts eigenstrains and thermal strains from total strains on integration points
  void eigstrmod ();

  /// reassambles total strains - adds eigenstrains and thermal strains to strains on integration points
  void totstrains ();
  
  /// computes thermal dilatancy matrix
  void matdilat (matrix &d,long ipp);

  /// stores appropriate volume of element belonging to given int. point
  void storeipvol (long ipp,double vol);
  
  /// computes averaged local values for non-loacl material models
  void nonlocaverage (long ipp,long im=0,long ido=0);

  /// returns the number of averaged local values.
  long give_num_averq(long ipp,long im);

  /// returns vector of averaged quantities
  void give_aver_quantv(long ipp,long im,long ido, vector &qv);

  /// returns radius of averaged neighbourhood for non-local material models
  double nonlocradius (long ipp,long im);

  /// returns id of the first non-local material model defined in the given int. point
  long givenonlocid(long ipp);

  /// updates internal variables for all int. points for the attained equilibrium state 
  void updateipval (void);

  /// updates internal variables for given int. point after attaining of equilibrium state 
  void updateipvalmat (long ipp,long im,long ido);

  /// sets all arrays on int. points to zero
  void clean_ip ();

  /// check time step size requirements which originate in material models for all integration points
  double dstep_red_mat(long im=0, long ido=0);

  /// check time step size requirements which originate in material models for the given ip
  double dstep_red_ip(long ipp, long im, long ido);

  
  // **********************************************************
  // Functions used for handling of stress return in plasticity
  // **********************************************************

  /// retruns actual value of yield function for plasticity materials
  double yieldfunction (long ipp, long idpm, matrix &sig, vector &q);

  /// retruns derivative of yield function with respect to stress tensor
  void dfdsigma (long ipp, long idpm, matrix &sig, vector &q, vector &dfds);

  /// retruns derivative of plastic potential with respect to stress tensor
  void dgdsigma (long ipp, long idpm, matrix &sig, vector &q, vector &dgds);

  /// retruns the second derivative of yield function with respect to stress tensor
  void dfdsigmadsigma (long ipp, long idpm, matrix &sig, matrix &dfdsds);

  /// retruns the second derivative of plastic potential with respect to stress tensor
  void dgdsigmadsigma (long ipp, long idpm, matrix &sig, matrix &dgdsds);

  /// retruns derivative of yield function with respect to %vector of hardening parameters
  void dfdqpar (long ipp, long idpm,matrix &sig,vector &q, vector &dq);

  /// retruns derivative of plastic potential with respect to %vector of hardening parameters
  void dgdqpar (long ipp, long idpm,matrix &sig,vector &q, vector &dq);

  /// retruns the second derivative of yield function with respect to stress tensor and %vector of hardening parameters
  void dfdsigmadq (long ipp, long idpm,matrix &sig,vector &q,matrix &dfdsdq);

  /// retruns the second derivative of plastic potential with respect to stress tensor and %vector of hardening parameters
  void dgdsigmadq (long ipp, long idpm,matrix &sig,vector &q,matrix &dgdsdq);

  /// derivative of hardening function with respect to stress tensor
  void dhdsigma (long ipp,long idpm,matrix &sigt,vector &q,matrix &dhds);

  /// derivative of hardening function with respect to consistency parameter 
  void dhdgamma (long ipp,long idpm,vector &epsp,matrix &sig,vector &q,vector &dhdc);

  /// derivative of hardening function with respect to %vector of hardening parameters
  void dhdqpar (long ipp, long idpm, long ido, matrix &sigt, double dgamma, vector &q, matrix &dhdq);

  /// retruns the actual value of  hardening/softening function
  void hardsoftfunction (long ipp,long idpm,matrix &sigt,vector &q,vector &h);

  /// returns %matrix of plastic moduli
  void plasticmoduli (long ipp,long idpm,vector &epsp,matrix &sig,matrix &h);

  /// returns contribution to denominator from hardening in cutting plane stress return algorithm
  void plasmodscalar(double &denom,long ipp, long idpm, long ido,matrix &sig, vector &eps, vector &epsp, vector &qtr,double gamma);

  /// returns the number of internal(hardening) parameters for plasticity models  
  long give_num_interparam (long ipp,long idpm);

  /// returns vector of hardening parameters
  void give_interparam (long ipp,long im,long ido,vector &q);

  /// updates of hardening parameters 
  void updateq (long ipp, long idpm, long ido, double dgamma, vector &eps, vector &epsp, matrix &sig, vector &q);
  
  /// cutting plane stress return algorithm
  void cutting_plane (long ipp,long im,long ido,double &gamma,vector &epsn,vector &epsp,vector &q,long ni,double err);

  /// multi-surface cutting plane stress return algorithm
  void mult_surf_cutting_plane (long ipp,vector &epsn,vector &epsp,vector &gamma,vector &q);

  /// closest point projection stress return algorithm
  void newton_stress_return (long ipp,long im,long ido,double &gamma,vector &epsn,vector &epsp,vector &q,long ni,double err);

  /// closest point projection stress return algorithm
  void newton_stress_return_2 (long ipp,long im,long ido,double &gamma,vector &epsn,vector &epsp,vector &q,long ni,double err);

  /// detection of plasticity for adaptivity computations
  void refresh_plast (long im=0, long ido=0);
  //void dfunctdqdq (long ipp, long idpm, matrix &sig, vector &q, matrix &dfdqdq);
  //void store_interparam (long ipp,long im,long ido,vector &q);



  // *************************************************
  // Functions for handling of scalar damage materials
  // *************************************************

  /// computation of equivalent strain
  void damfuncpar(long ipp, long im, vector &eps, vector &kappa);

  /// computation of new damage parameter omega
  double damfunction(long ipp, long im, long ido,vector &kappa, vector &eps, vector &omegao);

  /// computation of intial elsatic strain for damage models
  double epsefunction(long ipp, long idem);

  /// solver for scalar damage models
  double scal_dam_sol (long ipp,long im,long ido,vector &eps,vector &kappa,vector &omegao,vector &sigma,matrix &d);
  


  // *************************************************
  // Functions for handling of visco-plastic materials
  // *************************************************

  /// returns stress increments due to visco-plastic strains on int. point for given load case
  void givestressincr (long lcid,long ipp,long im,long ido,long fi,vector &sig);

  //void storestressincr (long lcid,long ipp,long im,long ido,vector &sig);

  /// computation of stresses Perzyna type visco-plastic materials
  void stiff_deps_vispl (long ipp,long im,long ido,vector &sig,vector &q,double dt);

  /// computation of contribution to right hand side for creep models - obsolate, candidate for removal
  void stiff_eps (long ipp);



  // *************************************************
  // Functions connected with Len-Jons potential model
  // *************************************************
 
  /// returns the first derivative of lenjones potential
  double give_first_derivative (long ipp,double r);

  /// returns the second derivative of lenjones potential
  double give_second_derivative (long ipp,double r);


  // *****************************************
  // Funtions for backup of int. point content
  // *****************************************

  /// save content of int. points to one huge text file
  void save_intpoints_txt (FILE *aux, sel &selelems, sel *selother);

  /// save content of int. points to several text files according to stored quantity
  void save_intpoints_txt (long ni, sel &selelems, sel *selother);

  /// save content of int. points to one huge binary file
  void save_intpoints_bin (FILE *aux, sel &selelems, sel *selother);

  /// save content of int. points to several binary files according to stored quantity
  void save_intpoints_bin (long ni, sel &selelems, sel *selother);

  /// restore content of int. points to one huge text file
  void restore_intpoints_txt (FILE *aux, sel &selelems, sel *selother, long **selid);

  /// restore content of int. points to several text files according to stored quantity
  void restore_intpoints_txt (sel &selelems, sel *selother, long **selid);

  /// restore content of int. points to one huge binary file
  void restore_intpoints_bin (FILE *aux, sel &selelems, sel *selother, long **selid);

  /// restore content of int. points to several binary files according to stored quantity
  void restore_intpoints_bin (sel &selelems, sel *selother, long **selid);


  
  

  
  ///  indicator of plastic regime in domain
  long plast;
  
  ///  type of eigstrain
  eigstraintype est;
  
  ///  number of material types
  long nmt;
  ///  total number of all integration points
  long tnip;
  ///  integration points
  intpoints *ip;
  ///  integration point - element correspondation
  ///  elip[number of int point] = number of element
  long *elip;

  ///  pointers to local coordinate systems in integration points
  long *lcsip;
  ///  array containing local coordinate system basis for integration points
  double **lcsbase;

  /// array with initial conditions at integration points (allocated at mechbclc::readinic)
  double **ic;
  
  ///  array containing volume appropriate to integration points
  double *ipv;
  /// maximum number of component of strain/stress arrays at nodes
  long max_ncompstrn;
  /// maximum number of component of strain/stress array on elements
  long max_ncompstre;
  /// maximum number of component of other array at nodes
  long max_nncompo;
  /// maximum number of component of eqother array on elements
  long max_encompo;

  ///  array containing eigenstrains id
  ///  the array is allocated in function mechbclc::read
  long **eigstrid;
  ///  array containing eigenstrains
  ///  the array is allocated only for several problems
  ///  eigstrains[number of int. point][number of component]
  ///  the array is allocated in function mechbclc::read
  double **eigstrains;
  ///  array containing eigenstresses
  ///  the array is allocated only for several problems
  ///  eigstresses[number of int. point][number of component]
  ///  the array is allocated in function mechbclc::read
  double **eigstresses;
  ///  array containing strains caused by temperature
  ///  tempstrains[number of int. point][number of component]
  double **tempstrains;
    

  ///  array containing non-mechanical quantities used for coupled problems
  ///  nonmechq[quantity no.][number of integ. point] - nonmechanical quantity at the required integration point
  double **nonmechq;
  /// number of actually used non-mechanical quantities (it is set by METR or according to temperature in the input file)
  long nnmq;
  /// array with ordering of used non-mechanical quantities (its size is nnmq)
  nonmechquant *nmqo;
  /// total number of known(=implemented) non-mechanical quantities
  static const long tnknmq = sizeof(nonmechquantstr)/sizeof(*nonmechquantstr);
  /// array of indices in nonmechq for particular non-mechanical quantities
  long nmqid[tnknmq];

  ///  elastic isotropic linear material
  elastisomat *eliso;
  ///  elastic general 3D material
  elastgmat3d *elgm3d;

  ///  simple plastic 1D material
  splas1d *spl1d;
  ///  J2 flow material
  j2flow *j2f;
  j2flow2 *j2f2;
  ///  Mohr-Coulomb material
  mohrcoulomb *mcoul;
  ///  Parabolic Mohr-Coulomb material
  mohrcoulombpar *mcpar;
  ///  Boer model of plasticity
  boermat *boerm;
  ///  Drucker-Prager model of plasticity
  drprag *drprm;
  ///  modified cam-clay model
  camclay *cclay;
  ///  modified cam-clay model coupled with moisture transfer
  camclaycoup *cclayc;
  ///  Sheffield plasticity model
  shefplast *shpl;
  ///  HISS plasticity model (Desai model)
  hissplas *hisspl;
  /// Glasgow scalar damage with a thermal influence  
  glasgmech *glasgmat;
  /// Glasgow damage model
  glasgowdam *glasgdam;
  ///  artificial model for combination of damage and creep model
  creepdam *crdam;
  ///  artificial model for combination of several time dependent material models switched on in the given time 
  timeswmat *tswmat;
  /// artificial material for effective stress concept   
  effstress *effstr;
  ///  microplane M4
  microM4 *mpM4;
  /// simplifed version of microplane
  microSIM *mpSIM;
  ///
  microfiber *mpfib;
  
  ///  simple model of viscosity
  simviscous *svis;
  ///  simple visco-plastic material
  svisplas *svipl;
  ///  Lemaitre visco-plastic model
  lemaitre *lmtr;

  ///  isotropic damage material model
  scaldam *scdam;

  ///  simple damage 3D material with crack closure
  scaldamcc *scdamcc;

  ///  ortotropic damage material model
  ortodam *ortdam;
  ///  ortotropic damage material model with crack rotation
  ortodamrot *ortdamrot;
  ///  anisotropic damage material model
  anisodam *anidam;
  ///  anisotropic damage material model with crack rotation
  anisodamrot *anidamrot;
  ///  aci78 model for creep and shrinkage
  aci78 *aci78mod;
  ///  cebfip78 model for creep and shrinkage
  cebfip78 *cebfip78mod;

  /// material with explicit prescribed diagram force/displacement
  graphmat *grmat;

  /// material with variable elastic parameters E and nu
  varelastisomat *veliso;

  /// combination of plasticity and special kind of elasticity (different loading and unloading modulus)
  geoelastmat *geoel;

  ///  B3 model of creep based on Bazant theory
  b3mat *crb3;
  ///  retardation spectrum
  rspecmat *crrs;
  ///  Double power law model of creep based on Bazant theory
  dplmat *crdpl;
  ///  model of creep based on Bazant theory
  creepb *crbaz;
  ///  model of creep based on effective Young modulus
  creep_effym *creffym;
  ///  consolidation model
  consol *csol;
  ///  Winkler-Pasternak material model of subsoil
  winpast *wpast;

  nonlocmicroM4 *nmpM4;

  ///  thermal isotropic dilatancy model
  therisomat *tidilat;
  
  ///  model of stress relaxation based on Eurocode 2
  relaxeuroc *relaxec;
  
  /// nonlocal plasticity models
  nonlocplast *nlplast;

  /// nonlocal damage models
  nonlocdamg *nldamg;

  ///  artificial model for combination of damage and plasticity model
  damplast *dampl;
  ///
  visplast *visplas;
  
  ///  Chen model of plasticity of concrete
  chen *chplast;

  ///  strain/stress points
  aepoints stra,stre;
  
  ///  Lennard-Jones interatomic potential
  lenjonesmat *lenjon;

  ///  material model for contact problems
  contactmat *conmat;

  ///  material model for contact problems according to MC90
  cebfipcontactmat *cebfipconmat;

  /// hypoplastic material for unsaturated soils
  hypoplast *hypopl;

  ///  material types
  mattype *mtype;
  ///  numbers of particular material types
  long *numtype;
  
  ///  material for shrinkage strain computation
  shrinkmat *shmat;

};

#endif
