#include "isotrmatc.h"
#include "stochdrivert.h"

isotrmatc::isotrmatc (void)
{
  k=0.0;  c=0.0;
}
isotrmatc::~isotrmatc (void)
{

}

void isotrmatc::read (XFILE *in)
{
  xfscanf (in,"%lf %lf %lf %lf %lf",&c,&k,&e,&nu,&alpha);
}

/**
   function creates conductivity matrix of the
   isotropic material

   @retval k - heat conductivity matrix of the isotropic
*/

double isotrmatc::get_k()
{
  return(k);
}

/**
   function creates capacity matrix of the
   isotropic material
   
   @retval c - heat capacity matrix of the isotropic
*/
double isotrmatc::get_c()
{
  return(c);
}

/**
   function creates Young's modulus
   isotropic material
   
   @retval e - Young's modulus
*/
double isotrmatc::get_e()
{
  return(e);
}

/**
   function creates Young's modulus
   isotropic material
   
   @retval nu - Poisson's constant
*/
double isotrmatc::get_nu()
{
  return(nu);
}

/**
   function creates thermal dilatation coefficient
   isotropic material
   
   @retval alpha - thermal dilatation coefficient
*/
double isotrmatc::get_alpha()
{
  return(alpha);
}

void isotrmatc::changeparam (atsel &atm,vector &val)
{
  long i;
  
  for (i=0;i<atm.num;i++){
    switch (atm.atrib[i]){
    case 0:{
      c=val[0];
      break;
    }
    case 1:{
      k=val[1];
      break;
    }
    case 2:{
      alpha=val[2];
      break;
    }
    default:{
      fprintf (stderr,"\n\n wrong number of atribute in function changeparam (%s, line %d).\n",__FILE__,__LINE__);
    }
    }
  }
}
